def l1_comparison(a1, a2):

    return sum([abs(x-y) for x,y in zip(a1,a2)])/len(a1)

def shock_velocity(v0,g):

    return (g-1.0)/2.0

def calc_analytic_d(d0,v0,g,r,t):

    vs = shock_velocity(v0,g)

    if r>vs*t:
        return d0*(1.0+v0*t/r)
    else:
        return d0*((g+1.0)/(g-1.0))**2

def calc_analytic_v(d0,v0,g,r,t):

    vs = shock_velocity(v0,g)

    if r>vs*t:
        return v0
    else:
        return 0

def calc_analytic_p(d0,v0,g,r,t):

    vs = shock_velocity(v0,g)

    if r>vs*t:
        return 0
    else:
        return 0.5*d0*v0**2*(g+1.0)**2/(g-1.0)

def consolidate_data_single(fname):

    import h5py
    import numpy

    f = h5py.File(fname)

    res = {}
    for field in f['geometry']:
	    res[field] = numpy.array(f['geometry'][field])
    for field in f['hydrodynamic']:
	    res[field] = numpy.array(f['hydrodynamic'][field])
    res['time'] = numpy.array(f['time'])
    return res

def consolidate_data(flist):

    import numpy

    clist = [consolidate_data_single(fname) for fname in flist]

    res = {}
    for field in clist[0]:
        res[field] = numpy.concatenate([itm[field] for itm in clist])
    return res

def main():

    import numpy
    import math
    import h5py
    import glob

 
    data = consolidate_data(glob.glob('final*.h5'))
    data['radius'] = numpy.sqrt(data['x_coordinate']**2+
                                data['y_coordinate']**2)
    data['speed'] = numpy.sqrt(data['x_velocity']**2+
                               data['y_velocity']**2)
    t = data['time'][0]
    g = numpy.loadtxt('adiabatic_index.txt')
    vc = numpy.loadtxt('collapse_velocity.txt')
    di = numpy.loadtxt('initial_density.txt')

    analytic_d = [calc_analytic_d(di,vc,g,r,t) for r in data['radius']]
    analytic_p = [calc_analytic_p(di,vc,g,r,t) for r in data['radius']]
    analytic_v = [calc_analytic_v(di,vc,g,r,t) for r in data['radius']]

    l1_density = l1_comparison(analytic_d, data['density'])
    l1_pressure = l1_comparison(analytic_p, data['pressure'])
    l1_velocity = l1_comparison(analytic_v, data['speed'])

    f = open('gradesheet.txt','w')
    f.write(str(l1_density)+'\n')
    f.write(str(l1_pressure)+'\n')
    f.write(str(l1_velocity)+'\n')
    f.close()

    return l1_density<1.24 and \
        l1_pressure<0.37 and \
        l1_velocity<0.077

if __name__ == '__main__':

    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')




import numpy
import cmath

def sgn(x):

    if x==0:
        return 0
    else:
        return x/abs(x)

def single_mode(x,t,k,omega,v,g0,dx,xi):

    return cmath.exp(1j*k*x)*g0*dx*xi*\
        (cmath.exp(1j*omega*t)-\
             (1-(1-cmath.exp(-1j*k*dx))*xi)**(t*v/xi/dx))/\
             (v*(-1+cmath.exp(1j*dx*xi*omega/v)+\
                      xi-xi*cmath.exp(-1j*k*dx)))

def primitives_to_conserved(hydro_data):
    """
    Converts primitive variables to variables that are conserved along streamlines 
    """

    g = hydro_data['adiabatic index']
    sound_speed = g*hydro_data['ambient']['pressure']/hydro_data['ambient']['density']
    res = {}
    res['positive riemann invariant'] = [dv + dp/(sound_speed*hydro_data['ambient']['density'])
                                         for dp, dv in zip(hydro_data['pert']['pressure'],
                                                           hydro_data['pert']['velocity'])]
    res['negative riemann invariant'] = [dv - dp/(sound_speed*hydro_data['ambient']['density'])
                                         for dp, dv in zip(hydro_data['pert']['pressure'],
                                                           hydro_data['pert']['velocity'])]
    res['entropy'] = [dp/hydro_data['ambient']['pressure'] - g*dd/hydro_data['ambient']['density']
                      for dd, dp in zip(hydro_data['pert']['density'],
                                        hydro_data['pert']['pressure'])]

    return res

def conserved_to_primitives(initial, conserved):

    import math

    g = initial['adiabatic index']
    res = {}
    res['velocity'] = [0.5*(jp+jm) for jp, jm in zip(conserved['positive riemann invariant'],
                                                     conserved['negative riemann invariant'])]
    sound_speed = math.sqrt(g*initial['ambient']['pressure']/initial['ambient']['density'])
    res['pressure'] = [0.5*sound_speed*initial['ambient']['density']*(jp-jm) 
                       for jp, jm in zip(conserved['positive riemann invariant'],
                                         conserved['negative riemann invariant'])]
    res['density'] = [(initial['ambient']['density']/g)*(dp/initial['ambient']['pressure']-ds)
                      for dp,ds in zip(res['pressure'],conserved['entropy'])]
    return res

def to_k_space(x_list, y_list):

    import numpy
    
    fy_list = numpy.fft.fft(y_list)
    dx = x_list[1] - x_list[0]
    k_list = 2*numpy.pi*numpy.fft.fftfreq(len(y_list),d=dx)
    return k_list, fy_list

def apply_filter(x_list, y_list, filter_func):

    k_list, fy_list = to_k_space(x_list, y_list)
    filter_list = [filter_func(k) for k in k_list]
    return numpy.fft.ifft([f*fy for f,fy in zip(filter_list, fy_list)])

def afd_advance_1(x_list, y_list, v, t, cfl=0.3):

    dx = x_list[1] - x_list[0]
    
    def filter_func(k):

        import cmath

        temp = 1 - cfl*(1.0-cmath.exp(-1j*k*dx*sgn(v)))
        return temp**(t*abs(v)/(cfl*dx))

    return [x.real for x in apply_filter(x_list, y_list, filter_func)]

def afd_advance_2(x_list, y_list, v, t, cfl=0.3):

    dx = x_list[1] - x_list[0]

    def filter_func(k):

        import cmath

        temp = 0.25*(4-cfl**2+cfl**2*cmath.cos(2*k*dx))-1j*cfl*cmath.sin(k*dx)
        return temp**(t*v/(cfl*dx))

    return [x.real for x in apply_filter(x_list, y_list, filter_func)]

def exact_advance(x_list, y_list, v, t):

    import cmath

    dx = x_list[1] - x_list[0]

    def filter_func(k):

        return cmath.exp(-1j*k*v*t)

    return [x.real for x in apply_filter(x_list, y_list, filter_func)]

def calc_propagation_speeds(initial):

    import math

    g = initial['adiabatic index']
    sound_speed = math.sqrt(g*initial['ambient']['pressure']/initial['ambient']['density'])
    return {'positive riemann invariant':initial['ambient']['velocity']+sound_speed,
            'negative riemann invariant':initial['ambient']['velocity']-sound_speed,
            'entropy':initial['ambient']['velocity']}

def time_advance(initial, time, scheme):

    initial_conserved = primitives_to_conserved(initial)
    propagation_speeds = calc_propagation_speeds(initial)
    final_conserved = {}
    for field in initial_conserved:
        final_conserved[field] = scheme(initial['grid'],
                                        initial_conserved[field],
                                        propagation_speeds[field],
                                        time)
    return conserved_to_primitives(initial, final_conserved)
    
def exact_time_advance(initial, time):

    return time_advance(initial, time, exact_advance)

def first_order_time_advance(initial, time, cfl=0.3):

    def my_scheme(x_list ,y_list, v, t):

        return afd_advance_1(x_list ,y_list, v, t, cfl)

    return time_advance(initial, time, my_scheme)

def second_order_time_advance(initial, time, cfl=0.3):

    def my_scheme(x_list ,y_list, v, t):

        return afd_advance_2(x_list ,y_list, v, t, cfl)

    return time_advance(initial, time, my_scheme)



import pylab
import numpy
import re
import sys
import Tkinter
import glob

pref = re.sub(r'/[^/]*$','',sys.argv[0])

n = len(glob.glob(pref+'/center_list.*.txt'))

if len(sys.argv)>1:
    val = int(sys.argv[1])
else:
    val = n

edge_list = numpy.loadtxt(pref+'/edge_list.'+str(val)+'.txt')
mass_flux_list = numpy.loadtxt(pref+'/mass_flux_list.'+str(val)+'.txt')
xmom_flux_list = numpy.loadtxt(pref+'/x_mom_flux_list.'+str(val)+'.txt')
ymom_flux_list = numpy.loadtxt(pref+'/y_mom_flux_list.'+str(val)+'.txt')
enr_flux_list = numpy.loadtxt(pref+'/enr_flux_list.'+str(val)+'.txt')

pylab.subplot(221)
pylab.plot(edge_list,
           mass_flux_list)
pylab.ylabel('Mass flux')
    
pylab.subplot(222)
pylab.plot(edge_list,
           xmom_flux_list)
pylab.ylabel('x momentum flux')

pylab.subplot(223)
pylab.plot(edge_list,
           ymom_flux_list)
pylab.ylabel('y momentum flux')

pylab.subplot(224)
pylab.plot(edge_list,
           enr_flux_list)
pylab.ylabel('energy flux')

pylab.show()



#! /usr/bin/python

def l1_fit(a1,a2):

    abs_dif = [abs(x-y) for x,y in zip(a1,a2)]
    return sum(abs_dif)/len(abs_dif)

import numpy

def main():

    import h5py

    graphic_flag = False

    # Numeric profiles
    h5f = h5py.File('final.h5','r')
    center_list = h5f['grid']
    density_list = h5f['density']
    pressure_list = h5f['pressure']
    velocity_list = h5f['x_velocity']
    time = numpy.array(h5f['time'])[0]

    # Analyic profiles
    gamma = 5./3.
    upstream_speed = 1
    shock_speed = (gamma-1)*upstream_speed/2
    shock_position = shock_speed*time
    upstream_density = 1
    downstream_density = (gamma+1)*upstream_density/(gamma-1)
    upstream_velocity = -1
    downstream_velocity = 0
    upstream_pressure = 1e-6
    downstream_pressure = ((gamma+1)/2)*upstream_density*\
        (downstream_velocity-upstream_velocity)**2
    
    def density_profile(x):
        if x<shock_position:
            return downstream_density
        else:
            return upstream_density

    def pressure_profile(x):
        if x<shock_position:
            return downstream_pressure
        else:
            return upstream_pressure

    def velocity_profile(x):
        if x<shock_position:
            return downstream_velocity
        else:
            return upstream_velocity

    analytic_density = [density_profile(x)
                        for x in center_list]
    analytic_pressure = [pressure_profile(x)
                        for x in center_list]
    analytic_velocity = [velocity_profile(x)
                        for x in center_list]

    if graphic_flag:
        import pylab

        pylab.subplot(311)
        pylab.plot(center_list,density_list)
        pylab.plot(center_list,analytic_density)

        pylab.subplot(312)
        pylab.plot(center_list,pressure_list)
        pylab.plot(center_list,analytic_pressure)

        pylab.subplot(313)
        pylab.plot(center_list,velocity_list)
        pylab.plot(center_list,analytic_velocity)

        pylab.show()

    l1_density = l1_fit(analytic_density,density_list)
    l1_pressure = l1_fit(analytic_pressure,pressure_list)
    l1_velocity = l1_fit(analytic_velocity,velocity_list)

    cond1 = l1_density<1e-2
    cond2 = l1_pressure<1e-2
    cond3 = l1_velocity<1e-2

    f = open('gradesheet.txt','w')
    f.write(str(l1_density)+'\n')
    f.write(str(l1_pressure)+'\n')
    f.write(str(l1_velocity)+'\n')
    f.close()

    return cond1 and cond2 and cond3

if __name__=='__main__':

    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')



def dist_sqr(arr):

    import numpy

    assert(len(arr)==2)

    return sum(numpy.array(arr)**2)

def main():

    import numpy

    threshold = 1e-6

    raw = numpy.loadtxt('output.txt')

    ref = [[2,3],
           [1,0],
           [-1,0],
           [1,0],
           [2,0],
           [0,1],
           [0,-1],
           [0,1],
           [0,2]]

    for n,a in zip(raw,ref):
        if dist_sqr(n-a)>threshold:
            return False

    return True

if __name__ == '__main__':

    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')
    



def calc_error_norm(a1,a2):

    abs_dif = [abs(x-y) for x,y in zip(a1,a2)]
    return sum(abs_dif)/len(a1)

def read_x_prof(fname):

    import numpy
    rawd = numpy.loadtxt(fname)
    return {'coordinate': rawd[:,0],
            'density': rawd[:,1],
            'pressure': rawd[:,2],
            'velocity': rawd[:,3],
            'sound speed': rawd[:,4]}

def calc_sound_speed(d,p,g):

    import math

    return math.sqrt(g*p/d)

class SineWave:

    def __init__(self,
                 amplitude,
                 wavelength,
                 phase,
                 offset):

        import math

        self.amp = amplitude
        self.k = 2*math.pi/float(wavelength)
        self.ph = phase
        self.offset = offset

    def eval(self, x):

        import math

        amp = self.amp
        k = self.k
        ph = self.ph
        offset = self.offset
        return amp*math.sin(k*x+ph)+offset

class SpatProf:

    def __init__(self, d0, p0, g,
                 dd, l):

        c0 = calc_sound_speed(d0,p0,g)
        self.c0 = c0
        self.density = SineWave(dd,l,0,d0)
        self.pressure = SineWave(dd*c0**2,
                                 l,0,p0)
        self.velocity = SineWave(dd*c0/d0,
                                 l,0,0)

def main():

    import numpy

    init_cond = \
        read_x_prof('x_prof_initial.txt')
    time = numpy.loadtxt('time.txt')
    final_cond = \
        read_x_prof('x_prof_final.txt')
    spat_prof = SpatProf(1.,3./5.,5./3.,1e-6,1.)
    analytic = dict()
    analytic['density'] = [spat_prof.density.eval(x-spat_prof.c0*time) \
                            for x in final_cond['coordinate']]
    analytic['pressure'] = [spat_prof.pressure.eval(x-spat_prof.c0*time) \
                            for x in final_cond['coordinate']]
    analytic['velocity'] = [spat_prof.velocity.eval(x-spat_prof.c0*time) \
                            for x in final_cond['coordinate']]

    if False:

        import pylab

        for i,v in enumerate(analytic):

            pylab.subplot(3,1,i+1)
            pylab.plot(final_cond['coordinate'],
                       final_cond[v],'.')
            pylab.plot(final_cond['coordinate'],
                       analytic[v],'.')
            pylab.ylabel(v)

        pylab.ylabel('x')
        pylab.show()

    error_norm = dict()
    for i in analytic.keys():
        error_norm[i] = calc_error_norm(final_cond[i],analytic[i])

    f = open('gradesheet.txt','w')
    for field in ['density','pressure','velocity']:
        f.write(str(error_norm[i])+'\n')
    f.close()

    return error_norm['density']<5e-8 and \
        error_norm['pressure']<5e-8 and \
        error_norm['velocity']<5e-8

if __name__=='__main__':
    print main()



import math
import logging

logging.basicConfig(filename="test_log.txt",level=logging.INFO)

def l1_test(a1,a2):

    return sum([abs(x-y) for x,y in zip(a1,a2)])/len(a1)

drift_velocity = 1

def density_prof(x,t):

    v = drift_velocity
    return 2+math.sin(2*math.pi*(x-v*t))

def pressure_prof(x):

    return 1

def velocity_prof(x):

    return 1

def main():

    import numpy

    logging.debug('Start of the test')

    rawd = numpy.loadtxt("sim_results.txt")

    logging.debug('Finished reading data from simulation')

    numeric = {'time':numpy.loadtxt('time.txt'),
               'coordinate':rawd[:,0],
               'density':rawd[:,1],
               'pressure':rawd[:,2],
               'velocity':rawd[:,3]}

    logging.debug('Finished putting simulation data in dictionary')

    analytic = {'density':[density_prof(x,numeric['time']) 
                           for x in numeric['coordinate']],
                'pressure':[pressure_prof(x) for x in numeric['coordinate']],
                'velocity':[velocity_prof(x) for x in numeric['coordinate']]}

    logging.debug('Finished calculating analytic profiles')

    if False:
        import pylab

        for i,field in enumerate(analytic):
            pylab.subplot(3,1,i+1)
            pylab.plot(numeric['coordinate'],
                       numeric[field])
            pylab.plot(numeric['coordinate'],
                       analytic[field])
            pylab.ylabel(field)

        pylab.xlabel('Distance')
        pylab.show()

    gof = {}
    for field in analytic:
        gof[field] = l1_test(numeric[field],analytic[field])

    f = open('gradesheet.txt','w')
    for field in ['density','pressure','velocity']:
        f.write(str(gof[field])+'\n')
    f.close()

    return gof['density']<0.1 and gof['pressure']<0.1 and gof['velocity']<0.1

if __name__=='__main__':

    print main()



import h5py
import pylab
import numpy
import matplotlib
import sys
import argparse

parser = argparse.ArgumentParser(description='Displays snapshot of the hydrodynamic simualation')
parser.add_argument("file_name",
                    help="path to snapshot file")
parser.add_argument("field",
                    help="Name of hydrodynamic variable")
args = parser.parse_args()

h5f = h5py.File(args.file_name)
x_list = numpy.array(h5f['x_coordinate'])
print len(x_list)
y_list = h5f['y_coordinate']
z_list = h5f[args.field]
vert_x_raw = numpy.array(h5f['x position of vertices'])
vert_y_raw = numpy.array(h5f['y position of vertices'])
vert_n_raw = h5f['Number of vertices in cell']
vert_idx_list = numpy.cumsum(vert_n_raw)

z_range = max(z_list) - min(z_list) + 1e-15
z_min = min(z_list)
z_scaled = [(z-z_min)/z_range for z in z_list]

for i in range(len(vert_n_raw)):
    upbound = vert_idx_list[i]
    if i==0:
        lowbound = 0
    else:
        lowbound = vert_idx_list[i-1]
    pylab.fill(vert_x_raw[lowbound:upbound],
               vert_y_raw[lowbound:upbound],
               fc=matplotlib.cm.jet(z_scaled[i]),
               ec='None')

pylab.title(args.field)
ax, _ = matplotlib.colorbar.make_axes(pylab.gca())
matplotlib.colorbar.ColorbarBase(ax,norm=matplotlib.colors.Normalize(vmin=min(z_list),vmax=max(z_list)))
pylab.show()



#! /usr/bin/python

def goodness_of_fit(a1, a2):

    import math

    diff2 = [(x-y)**2 for x,y in zip(a1,a2)]
    return math.sqrt(sum(diff2)/(max(a1)-min(a1))/len(a1))

def main():

    import numpy
    import os
    import imp
    import h5py
    import glob
    enrs = imp.load_source('enrs',os.environ['RICH_ROOT']+'/analytic/enrs.py')

    ns = len(glob.glob('process_*_final.h5'))
    if ns>0:
        x = []
        d = []
        p = []
        v = []
        for fname in glob.glob('process_*_final.h5'):
            f = h5py.File(fname,'r+')
            x.extend(f['x_coordinate'])
            d.extend(f['density'])
            p.extend(f['pressure'])
            v.extend(f['x_velocity'])
    else:
        h5f = h5py.File('final.h5')
        x = h5f['geometry']['x_coordinate']
        d = h5f['hydrodynamic']['density']
        p = h5f['hydrodynamic']['pressure']
        v = h5f['hydrodynamic']['x_velocity']

    xs = numpy.sort(x)
    ids = numpy.argsort(x)
    ds = [d[i] for i in ids]
    ps = [p[i] for i in ids]
    vs = [v[i] for i in ids]

    left = enrs.Primitive(1,2,0)
    right = enrs.Primitive(1,1,0)
    prof = enrs.RiemannProfile(left,right,5./3.)
    t = numpy.loadtxt('time.txt')
    offset = 0.5;
    da = [prof.CalcPrim((i-offset)/t).Density for i in xs]
    pa = [prof.CalcPrim((i-offset)/t).Pressure for i in xs]
    va = [prof.CalcPrim((i-offset)/t).Velocity for i in xs]

    gof1 = goodness_of_fit(ds,da)
    gof2 = goodness_of_fit(ps,pa)
    gof3 = goodness_of_fit(vs,va)

    f = open('gradesheet.txt','w')
    f.write(str(gof1)+'\n')
    f.write(str(gof2)+'\n')
    f.write(str(gof3)+'\n')
    f.close()
    
    return gof1<0.067 and gof2<0.053 and gof3<0.062

if __name__=='__main__':

    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')



#! /usr/bin/python

import matplotlib
matplotlib.use('Qt4Agg')
import pylab
import numpy
import sys

aux = sys.argv[0]
pref = aux[:aux.find('show_mesh_points.py')]

rawd = numpy.loadtxt(pref+'mesh_generating_points.txt')
pylab.plot(rawd[:,0], rawd[:,1],'x')

rawd2 = numpy.loadtxt(pref+'edges.txt')
for i in range(len(rawd2[:,0])):
    pylab.plot((rawd2[i,0],rawd2[i,2]),(rawd2[i,1],rawd2[i,3]),'r')
pylab.axis('equal')
pylab.xlim((-0.05, 1.05))
pylab.ylim((-0.05, 1.05))
pylab.show()




#! /usr/bin/python

def goodness_of_fit(a1, a2):

    import math

    diff2 = [(x-y)**2 for x,y in zip(a1,a2)]
    return math.sqrt(sum(diff2)/(max(a1)-min(a1))/len(a1))

def main():

    import numpy
    import h5py

    # Simulation results

    initial_data = h5py.File('initial.h5')
    xr0 = initial_data['geometry']['x_coordinate']
    dr0 = initial_data['hydrodynamic']['density']
    pr0 = initial_data['hydrodynamic']['pressure']
    vr0 = initial_data['hydrodynamic']['x_velocity']
    g = numpy.loadtxt('adiabatic_index.txt')
    cr0 = [numpy.sqrt(g*p/d) for d,p in zip(dr0,pr0)]

    final_data = h5py.File('final.h5')
    time = numpy.array(final_data['time'])[0]

    xs0 = numpy.sort(xr0)
    ids0 = numpy.argsort(xr0)
    ds0 = [dr0[i] for i in ids0]
    ps0 = [pr0[i] for i in ids0]
    vs0 = [vr0[i] for i in ids0]
    cs0 = [cr0[i] for i in ids0]

    xr = final_data['geometry']['x_coordinate']
    dr = final_data['hydrodynamic']['density']
    pr = final_data['hydrodynamic']['pressure']
    vr = final_data['hydrodynamic']['x_velocity']
    cr = [numpy.sqrt(g*p/d) for d,p in zip(dr,pr)]

    xs = numpy.sort(xr)
    ids = numpy.argsort(xr)
    ds = [dr[i] for i in ids]
    ps = [pr[i] for i in ids]
    vs = [vr[i] for i in ids]
    cs = [cr[i] for i in ids]
    
    # Analytic results
    xa = [x+time*(c+v) for x,v,c in zip(xs0,vs0,cs0)]

    # Prepare for interpolation
    x_inside = [z for z in xa if z<numpy.max(xs) and 
                z>numpy.min(xs)]
    d_analytic = [ds0[i] for i in range(len(xa)) 
                  if xa[i]<numpy.max(xs) and
                  xa[i]>numpy.min(xs)]
    p_analytic = [ps0[i] for i in range(len(xa)) 
                  if xa[i]<numpy.max(xs) and
                  xa[i]>numpy.min(xs)]
    v_analytic = [vs0[i] for i in range(len(xa)) 
                  if xa[i]<numpy.max(xs) and
                  xa[i]>numpy.min(xs)]

    #d_analytic = numpy.interp(x_inside,xa,ds0)
    d_numeric = numpy.interp(x_inside,xs,ds)
    #p_analytic = numpy.interp(x_inside,xa,ps0)
    p_numeric = numpy.interp(x_inside,xs,ps)
    #v_analytic = numpy.interp(x_inside,xa,vs0)
    v_numeric = numpy.interp(x_inside,xs,vs)

    gof1 = goodness_of_fit(d_numeric,d_analytic)
    gof2 = goodness_of_fit(p_numeric,p_analytic)
    gof3 = goodness_of_fit(v_numeric,v_analytic)

    f = open('gradesheet.txt','w')
    f.write(str(gof1)+'\n')
    f.write(str(gof2)+'\n')
    f.write(str(gof3)+'\n')
    f.close()

    return gof1<0.22 and \
        gof2<0.65 and \
        gof3<0.31

if __name__=='__main__':

    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')



#! /usr/bin/python

def goodness_of_fit(a1, a2):

    import math

    diff2 = [(x-y)**2 for x,y in zip(a1,a2)]
    return math.sqrt(sum(diff2)/(max(a1)-min(a1))**2/len(a1))

def main():

    import os
    import imp
    enrs = imp.load_source('enrs',os.environ['RICH_ROOT']+'/analytic/enrs.py')

    import numpy
    import h5py

    left = enrs.Primitive(1,10,0);
    right = enrs.Primitive(1,1,0);
    prof = enrs.RiemannProfile(left,right,5./3.)
    offset = 0.5

    h5file = h5py.File('final.h5','r')
    t = h5file['time']
    x = h5file['grid']
    d = h5file['density']
    p = h5file['pressure']
    v = h5file['x_velocity']
    da = [prof.CalcPrim((i-offset)/t).Density for i in x]
    pa = [prof.CalcPrim((i-offset)/t).Pressure for i in x]
    va = [prof.CalcPrim((i-offset)/t).Velocity for i in x]

    gof1 = goodness_of_fit(d,da)
    gof2 = goodness_of_fit(p,pa)
    gof3 = goodness_of_fit(v,va)

    f = open('gradesheet.txt','w')
    f.write(str(gof1)+'\n')
    f.write(str(gof2)+'\n')
    f.write(str(gof3)+'\n')
    f.close()

    return gof1<0.14 and gof2<0.06 and gof3 < 0.13

if __name__=='__main__':
    
    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')




import pylab
import numpy
import re
import sys
import Tkinter
import glob

pref = re.sub(r'/[^/]*$','',sys.argv[0])

n = len(glob.glob(pref+'/center_list.[0-9]+.txt'))

if len(sys.argv)>1:
    val = int(sys.argv[1])
else:
    val = n

x_list = numpy.loadtxt(pref+'/center_list.'+str(val)+'.txt')
d_list = numpy.loadtxt(pref+'/density_list.'+str(val)+'.txt')
p_list = numpy.loadtxt(pref+'/pressure_list.'+str(val)+'.txt')
v_list = numpy.loadtxt(pref+'/xvelocity_list.'+str(val)+'.txt')

pylab.subplot(311)
pylab.plot(x_list,d_list)
pylab.ylabel('Density')
    
pylab.subplot(312)
pylab.plot(x_list,p_list)
pylab.ylabel('Pressure')

pylab.subplot(313)
pylab.plot(x_list,v_list)
pylab.ylabel('Velocity')
pylab.xlabel('Distance')

pylab.show()



import time
from threading import Thread

import pytest

import env  # noqa: F401
from pybind11_tests import callbacks as m


def test_callbacks():
    from functools import partial

    def func1():
        return "func1"

    def func2(a, b, c, d):
        return "func2", a, b, c, d

    def func3(a):
        return f"func3({a})"

    assert m.test_callback1(func1) == "func1"
    assert m.test_callback2(func2) == ("func2", "Hello", "x", True, 5)
    assert m.test_callback1(partial(func2, 1, 2, 3, 4)) == ("func2", 1, 2, 3, 4)
    assert m.test_callback1(partial(func3, "partial")) == "func3(partial)"
    assert m.test_callback3(lambda i: i + 1) == "func(43) = 44"

    f = m.test_callback4()
    assert f(43) == 44
    f = m.test_callback5()
    assert f(number=43) == 44


def test_bound_method_callback():
    # Bound Python method:
    class MyClass:
        def double(self, val):
            return 2 * val

    z = MyClass()
    assert m.test_callback3(z.double) == "func(43) = 86"

    z = m.CppBoundMethodTest()
    assert m.test_callback3(z.triple) == "func(43) = 129"


def test_keyword_args_and_generalized_unpacking():
    def f(*args, **kwargs):
        return args, kwargs

    assert m.test_tuple_unpacking(f) == (("positional", 1, 2, 3, 4, 5, 6), {})
    assert m.test_dict_unpacking(f) == (
        ("positional", 1),
        {"key": "value", "a": 1, "b": 2},
    )
    assert m.test_keyword_args(f) == ((), {"x": 10, "y": 20})
    assert m.test_unpacking_and_keywords1(f) == ((1, 2), {"c": 3, "d": 4})
    assert m.test_unpacking_and_keywords2(f) == (
        ("positional", 1, 2, 3, 4, 5),
        {"key": "value", "a": 1, "b": 2, "c": 3, "d": 4, "e": 5},
    )

    with pytest.raises(TypeError) as excinfo:
        m.test_unpacking_error1(f)
    assert "Got multiple values for keyword argument" in str(excinfo.value)

    with pytest.raises(TypeError) as excinfo:
        m.test_unpacking_error2(f)
    assert "Got multiple values for keyword argument" in str(excinfo.value)

    with pytest.raises(RuntimeError) as excinfo:
        m.test_arg_conversion_error1(f)
    assert "Unable to convert call argument" in str(excinfo.value)

    with pytest.raises(RuntimeError) as excinfo:
        m.test_arg_conversion_error2(f)
    assert "Unable to convert call argument" in str(excinfo.value)


def test_lambda_closure_cleanup():
    m.test_lambda_closure_cleanup()
    cstats = m.payload_cstats()
    assert cstats.alive() == 0
    assert cstats.copy_constructions == 1
    assert cstats.move_constructions >= 1


def test_cpp_callable_cleanup():
    alive_counts = m.test_cpp_callable_cleanup()
    assert alive_counts == [0, 1, 2, 1, 2, 1, 0]


def test_cpp_function_roundtrip():
    """Test if passing a function pointer from C++ -> Python -> C++ yields the original pointer"""

    assert (
        m.test_dummy_function(m.dummy_function) == "matches dummy_function: eval(1) = 2"
    )
    assert (
        m.test_dummy_function(m.roundtrip(m.dummy_function))
        == "matches dummy_function: eval(1) = 2"
    )
    assert (
        m.test_dummy_function(m.dummy_function_overloaded)
        == "matches dummy_function: eval(1) = 2"
    )
    assert m.roundtrip(None, expect_none=True) is None
    assert (
        m.test_dummy_function(lambda x: x + 2)
        == "can't convert to function pointer: eval(1) = 3"
    )

    with pytest.raises(TypeError) as excinfo:
        m.test_dummy_function(m.dummy_function2)
    assert "incompatible function arguments" in str(excinfo.value)

    with pytest.raises(TypeError) as excinfo:
        m.test_dummy_function(lambda x, y: x + y)
    assert any(
        s in str(excinfo.value)
        for s in ("missing 1 required positional argument", "takes exactly 2 arguments")
    )


def test_function_signatures(doc):
    assert doc(m.test_callback3) == "test_callback3(arg0: Callable[[int], int]) -> str"
    assert doc(m.test_callback4) == "test_callback4() -> Callable[[int], int]"


def test_movable_object():
    assert m.callback_with_movable(lambda _: None) is True


@pytest.mark.skipif(
    "env.PYPY",
    reason="PyPy segfaults on here. See discussion on #1413.",
)
def test_python_builtins():
    """Test if python builtins like sum() can be used as callbacks"""
    assert m.test_sum_builtin(sum, [1, 2, 3]) == 6
    assert m.test_sum_builtin(sum, []) == 0


def test_async_callbacks():
    # serves as state for async callback
    class Item:
        def __init__(self, value):
            self.value = value

    res = []

    # generate stateful lambda that will store result in `res`
    def gen_f():
        s = Item(3)
        return lambda j: res.append(s.value + j)

    # do some work async
    work = [1, 2, 3, 4]
    m.test_async_callback(gen_f(), work)
    # wait until work is done
    from time import sleep

    sleep(0.5)
    assert sum(res) == sum(x + 3 for x in work)


def test_async_async_callbacks():
    t = Thread(target=test_async_callbacks)
    t.start()
    t.join()


def test_callback_num_times():
    # Super-simple micro-benchmarking related to PR #2919.
    # Example runtimes (Intel Xeon 2.2GHz, fully optimized):
    #   num_millions  1, repeats  2:  0.1 secs
    #   num_millions 20, repeats 10: 11.5 secs
    one_million = 1000000
    num_millions = 1  # Try 20 for actual micro-benchmarking.
    repeats = 2  # Try 10.
    rates = []
    for rep in range(repeats):
        t0 = time.time()
        m.callback_num_times(lambda: None, num_millions * one_million)
        td = time.time() - t0
        rate = num_millions / td if td else 0
        rates.append(rate)
        if not rep:
            print()
        print(
            f"callback_num_times: {num_millions:d} million / {td:.3f} seconds = {rate:.3f} million / second"
        )
    if len(rates) > 1:
        print("Min    Mean   Max")
        print(f"{min(rates):6.3f} {sum(rates) / len(rates):6.3f} {max(rates):6.3f}")



#! /usr/bin/python

def goodness_of_fit(a1, a2):

    import numpy

    return sum(numpy.abs(numpy.array(a1)-numpy.array(a2)))/len(a1)

def consolidate_single(fname):

    import h5py
    import numpy

    f = h5py.File(fname,'r+')
    #res = {key:numpy.array(f[key]) for key in f}
    res = {}
    res['x_coordinate'] = numpy.array(f['geometry']['x_coordinate'])
    res['y_coordinate'] = numpy.array(f['geometry']['y_coordinate'])
    res['x_velocity'] = numpy.array(f['hydrodynamic']['x_velocity'])
    res['y_velocity'] = numpy.array(f['hydrodynamic']['y_velocity'])
    res['density'] = numpy.array(f['hydrodynamic']['density'])
    res['pressure'] = numpy.array(f['hydrodynamic']['pressure'])
    f.close()
    return res

def consolidate_multiple(f_list):

    assert(len(f_list)>1)

    import numpy

    parts_list = [consolidate_single(fname) for fname in f_list]
    return {key:numpy.concatenate([part[key] for part in parts_list]) for key in parts_list[0]}

def find_max_pos_par(x_list, y_list, window=2):

    import numpy

    mi = numpy.argmax(y_list)

    fit_data = numpy.polyfit(x_list[mi-window:mi+window+1],
                             y_list[mi-window:mi+window+1],
                             2)
    return -0.5*fit_data[1]/fit_data[0]

def calc_shock_front(numeric):

    import numpy

    radially_sorted = {field:numpy.array([x for (y,x) in sorted(zip(numeric['radius'],numeric[field]))])
                       for field in ['radius','density','pressure','velocity']}
    res = {'radius':find_max_pos_par(radially_sorted['radius'],
                                     radially_sorted['pressure'])}
    for field in ['density','pressure','velocity']:
        res[field] = numpy.interp(res['radius'],
                                  radially_sorted['radius'],
                                  radially_sorted[field])
    return res

class SedovTaylorProfiles:

    def __init__(self, upstream, shock_front, g, w, n ,nip=1000):

        import numpy
        import imp
        import os

        sedov_taylor = imp.load_source('sedov_taylor',
                                       os.environ['RICH_ROOT']+'/analytic/sedov_taylor.py')

        self.upstream = upstream
        ssv_list = numpy.linspace(1e-6+1./g,
                                  2./(g+1.),
                                  num=nip)
        self.shock_radius = shock_front['radius']
        self.tables = {'radius':numpy.array([shock_front['radius']*sedov_taylor.vtoz(v,w,g,n) for v in ssv_list]),
                       'density':numpy.array([shock_front['density']*sedov_taylor.vtod(v,w,g,n)/((g+1.)/(g-1.)) for v in ssv_list]),
                       'pressure':numpy.array([shock_front['pressure']*sedov_taylor.vtop(v,w,g,n)/(2./(g+1.)) for v in ssv_list]),
                       'velocity':numpy.array([shock_front['velocity']*sedov_taylor.vtoz(v,w,g,n)*v/(2./(g+1.)) for v in ssv_list])}

    def calc(self, field, r):

        import numpy

        if r>self.shock_radius:
            return self.upstream[field]
        else:
            return numpy.interp(r,
                                self.tables['radius'],
                                self.tables[field])

def main():

    import numpy
    import glob

    np = len(glob.glob('process_*_final.h5'))

    if np>0:
        numeric = consolidate_multiple(glob.glob('process_*_final.h5'))
    else:
        numeric = consolidate_single('final.h5')
    numeric['radius'] = numpy.sqrt(numeric['x_coordinate']**2+numeric['y_coordinate']**2)
    numeric['velocity'] = (numeric['x_coordinate']*numeric['x_velocity']+
                           numeric['y_coordinate']*numeric['y_velocity'])/numeric['radius']

    st_prof = SedovTaylorProfiles({'density':1,'pressure':0.01,'velocity':0},
                                  calc_shock_front(numeric),
                                  5./3.,0,2)
    analytic = {field:numpy.array([st_prof.calc(field,r) for r in numeric['radius']]) for field in ['density','pressure','velocity']}

    l1_data = {field:goodness_of_fit(analytic[field],
                                     numeric[field])
               for field in analytic}

    f = open('gradesheet.txt','w')
    for field in ['density','pressure','velocity']:
        f.write(str(l1_data[field])+'\n')
    f.close()

    if False:
        import pylab
        for n,f in enumerate(['density','pressure','velocity']):
            pylab.subplot(3,1,n+1)
            pylab.plot(numeric['radius'],
                       numeric[f],'.')
            pylab.plot(numeric['radius'],
                       analytic[f],'.')
        pylab.show()

    return l1_data['density']<0.5 and l1_data['pressure']<15 and l1_data['velocity']<1.5

import sys
if __name__=='__main__':

    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')
        



import pylab
import numpy
import re
import glob
import sys
import os

if __name__=='__main__':

    pref = re.sub(r'/[^/]*$','',sys.argv[0])

    for txtfile in glob.glob(pref+'/center_list.*.txt'):
        x_list = numpy.loadtxt(txtfile)
        d_list = numpy.loadtxt(txtfile.replace('center','density'))
        p_list = numpy.loadtxt(txtfile.replace('center','pressure'))
        v_list = numpy.loadtxt(txtfile.replace('center','xvelocity'))

        pylab.clf()

        pylab.subplot(311)
        pylab.plot(x_list,d_list)
        pylab.ylabel('Density')

        pylab.subplot(312)
        pylab.plot(x_list,p_list)
        pylab.ylabel('Pressure')

        pylab.subplot(313)
        pylab.plot(x_list,v_list)
        pylab.ylabel('Velocity')
        pylab.xlabel('Distance')

        pylab.savefig(txtfile.replace('center_list','snapshot').replace('txt','png'))



#! /usr/bin/python

def l1_fit_factor(a1, a2):

    import math

    diff2 = [abs(x-y) for x,y in zip(a1,a2)]
    return sum(diff2)/(max(a1)-min(a1))/len(a1)

def main():

    import numpy
    import os
    import imp
    import h5py

    h5f = h5py.File('final.h5','r+')
    x_list = h5f['geometry']['x_coordinate']
    d_list = h5f['hydrodynamic']['density']
    p_list = h5f['hydrodynamic']['pressure']
    vx_list = h5f['hydrodynamic']['x_velocity']

    rho_0 = 1.0
    amp = 0.001
    k = 2*numpy.pi/1.0
    v_0 = 0.1
    omega = k*v_0
    p_0 = 1.0
    gamma = 5./3.
    c_0 = numpy.sqrt(gamma*p_0/rho_0)
    t = numpy.loadtxt('time.txt')

    def density_exact(x):

        return rho_0-\
            (amp*rho_0*(2*c_0*numpy.sin(k*x) - (c_0 + v_0)*numpy.sin(k*(c_0*t - \
t*v_0 + x)) + (-c_0 + v_0)*numpy.sin(k*(-(t*(c_0 + v_0)) + \
x))))/(2.*c_0*k*(c_0 - v_0)*(c_0 + v_0))

    def pressure_exact(x):

        return p_0-\
            (amp*c_0*rho_0*(2*c_0*numpy.sin(k*x) + (-c_0 + \
v_0)*numpy.sin(k*(c_0*t - t*v_0 + x)) - (c_0 + \
v_0)*numpy.sin(k*(-(t*(c_0 + v_0)) + x))))/(2.*k*(c_0 - v_0)*(c_0 + \
v_0))

    def velocity_exact(x):

        return v_0-\
            (amp*(-2*v_0*numpy.sin(k*x) + (c_0 + v_0)*numpy.sin(k*(c_0*t - t*v_0 \
+ x)) + (-c_0 + v_0)*numpy.sin(k*(-(t*(c_0 + v_0)) + x))))/(2.*k*(c_0 \
- v_0)*(c_0 + v_0))

    d_exact = [density_exact(x)
               for x in x_list]
    p_exact = [pressure_exact(x)
               for x in x_list]
    v_exact = [velocity_exact(x)
               for x in x_list]

    if False:
        import pylab
        pylab.subplot(311)
        pylab.plot(x_list, d_list, '.')
        pylab.plot(x_list, d_exact, '.')
        pylab.subplot(312)
        pylab.plot(x_list, p_list, '.')
        pylab.plot(x_list, p_exact, '.')
        pylab.subplot(313)
        pylab.plot(x_list, vx_list, '.')
        pylab.plot(x_list, v_exact, '.')
        pylab.show()

    l1_density = l1_fit_factor(d_list, d_exact)
    l1_pressure = l1_fit_factor(p_list, p_exact)
    l1_velocity = l1_fit_factor(vx_list, v_exact)

    f = open('gradesheet.txt','w')
    f.write(str(l1_density)+'\n')
    f.write(str(l1_pressure)+'\n')
    f.write(str(l1_velocity)+'\n')
    f.close()
    
    return l1_density < 0.07 and \
        l1_pressure < 0.1 and \
        l1_velocity < 0.23

import sys
if __name__=='__main__':

    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')



#! /usr/bin/python

import matplotlib
matplotlib.use('Qt4Agg')
import pylab
import numpy
import sys
import imp

# Simulation results

pref = sys.argv[0].replace('draw_rad_prof.py','')

rawd = numpy.loadtxt(pref+'x_prof_initial.txt')
xr0 = rawd[:,0]
dr0 = rawd[:,1]
pr0 = rawd[:,2]
vr0 = rawd[:,3]
cr0 = rawd[:,4]
time = numpy.loadtxt(pref+'time.txt')

xs0 = numpy.sort(xr0)
ids0 = numpy.argsort(xr0)
ds0 = [dr0[i] for i in ids0]
ps0 = [pr0[i] for i in ids0]
vs0 = [vr0[i] for i in ids0]
cs0 = [cr0[i] for i in ids0]

rawd = numpy.loadtxt(pref+'x_prof_final.txt')
xr = rawd[:,0]
dr = rawd[:,1]
pr = rawd[:,2]
vr = rawd[:,3]
cr = rawd[:,4]

xs = numpy.sort(xr)
ids = numpy.argsort(xr)
ds = [dr[i] for i in ids]
ps = [pr[i] for i in ids]
vs = [vr[i] for i in ids]
cs = [cr[i] for i in ids]

# Analytic results
xa = [x+time*(c+v) for x,v,c in zip(xs0,vs0,cs0)]

# Show data
pylab.subplot(311)
pylab.plot(xs,ds,xa,ds0,xs0,ds0)
pylab.xlabel('r')
pylab.ylabel('Density')

pylab.subplot(312)
pylab.plot(xs,ps,xa,ps0,xs0,ps0)
pylab.xlabel('r')
pylab.ylabel('Pressure')

pylab.subplot(313)
pylab.plot(xs,vs,xa,vs0,xs0,vs0)
pylab.xlabel('r')
pylab.ylabel('Velocity')
pylab.legend(('Numeric','Analytic','Initial'))

pylab.show()



# Dummy value.
#
#  No texture, but the value to be used as 'texture semantic'
#  (#aiMaterialProperty::mSemantic) for all material properties
# # not* related to textures.
#
aiTextureType_NONE = 0x0

# The texture is combined with the result of the diffuse
#  lighting equation.
#
aiTextureType_DIFFUSE = 0x1

# The texture is combined with the result of the specular
#  lighting equation.
#
aiTextureType_SPECULAR = 0x2

# The texture is combined with the result of the ambient
#  lighting equation.
#
aiTextureType_AMBIENT = 0x3

# The texture is added to the result of the lighting
#  calculation. It isn't influenced by incoming light.
#
aiTextureType_EMISSIVE = 0x4

# The texture is a height map.
#
#  By convention, higher gray-scale values stand for
#  higher elevations from the base height.
#
aiTextureType_HEIGHT = 0x5

# The texture is a (tangent space) normal-map.
#
#  Again, there are several conventions for tangent-space
#  normal maps. Assimp does (intentionally) not
#  distinguish here.
#
aiTextureType_NORMALS = 0x6

# The texture defines the glossiness of the material.
#
#  The glossiness is in fact the exponent of the specular
#  (phong) lighting equation. Usually there is a conversion
#  function defined to map the linear color values in the
#  texture to a suitable exponent. Have fun.
#
aiTextureType_SHININESS = 0x7

# The texture defines per-pixel opacity.
#
#  Usually 'white' means opaque and 'black' means
#  'transparency'. Or quite the opposite. Have fun.
#
aiTextureType_OPACITY = 0x8

# Displacement texture
#
#  The exact purpose and format is application-dependent.
#  Higher color values stand for higher vertex displacements.
#
aiTextureType_DISPLACEMENT = 0x9

# Lightmap texture (aka Ambient Occlusion)
#
#  Both 'Lightmaps' and dedicated 'ambient occlusion maps' are
#  covered by this material property. The texture contains a
#  scaling value for the final color value of a pixel. Its
#  intensity is not affected by incoming light.
#
aiTextureType_LIGHTMAP = 0xA

# Reflection texture
#
# Contains the color of a perfect mirror reflection.
# Rarely used, almost never for real-time applications.
#
aiTextureType_REFLECTION = 0xB

# Unknown texture
#
#  A texture reference that does not match any of the definitions
#  above is considered to be 'unknown'. It is still imported
#  but is excluded from any further postprocessing.
#
aiTextureType_UNKNOWN = 0xC



#! /usr/bin/python

def main():

    import numpy
    
    rawd = numpy.loadtxt('res.txt')
    return rawd[0]==1 and \
        rawd[1]==2 and \
        rawd[2]==3 and \
        rawd[3]==4 and \
        rawd[4]==5 and \
        rawd[5]==6

if __name__=='__main__':
    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')




def gcc_strict_warning_flags():

    return '-Wfatal-errors '+\
        '-std=c++0x '+\
        '-Wmissing-declarations '+\
        '-Wendif-labels '+\
        '-Wabi '+\
        '-Wctor-dtor-privacy '+\
        '-Wstrict-null-sentinel '+\
        '-Winvalid-offsetof '+\
        '-Wsign-promo '+\
        '-Woverloaded-virtual '+\
        '-Wsynth '+\
        '-Wlogical-op '+\
        '-Wno-unused-result '+\
        '-Weffc++ '+\
        '-Wno-missing-braces '+\
        '-Wno-missing-field-initializers '+\
        '-Wformat=2 '+\
        '-Wswitch-default '+\
        '-Wswitch-enum '+\
        '-Wpointer-arith '+\
        '-Wshadow '+\
        '-pedantic-errors '+\
        '-Wcast-align '+\
        '-Wcast-qual '+\
        '-Wchar-subscripts '+\
        '-Wcomment '+\
        '-Wconversion '+\
        '-Wdisabled-optimization '+\
        '-Wformat-nonliteral -Wformat-security '+\
        '-Wformat-y2k '+\
        '-Wimport  -Winit-self '+\
        '-Winvalid-pch '+\
        '-Wundef '+\
        '-Wmissing-braces '+\
        '-Wmissing-field-initializers -Wmissing-format-attribute '+\
        '-Wmissing-include-dirs '+\
        '-Wpacked -Wparentheses '+\
        '-Wredundant-decls -Wreturn-type '+\
        '-Wsequence-point -Wsign-compare  -Wstack-protector '+\
        '-Wstrict-aliasing -Wstrict-aliasing=2 -Wswitch  -Wswitch-default '+\
        '-Wswitch-enum -Wtrigraphs  -Wuninitialized '+\
        '-Wunknown-pragmas '+\
        '-Wunused '+\
        '-Wunused-function  -Wunused-label  -Wunused-parameter '+\
        '-Wunused-value  -Wunused-variable  -Wvariadic-macros '+\
        '-Wvolatile-register-var  -Wwrite-strings '+\
        '-Wall -Wextra -pedantic -Werror'

def safe_create(folder_name):

    import os

    if not os.path.isdir(folder_name):
        os.mkdir(folder_name)

def get_all_files(absroot):

    import os

    res = []
    for root, dirs, files in os.walk(absroot):
        for name in files:
            res.append(os.path.join(root,name))
    return res

def get_source_files(absroot):

    return [fname for fname in get_all_files(absroot)
            if fname[-4:]=='.cpp']

def source2lib(fname, lname):

    return lname+'/'+fname.split('/')[-1].replace('.cpp','.o')

def strict_compile(fname, lname):

    import subprocess

    oname = source2lib(fname, lname)
    compile_command = ['g++','-c','-g',fname,'-O0']
    for item in gcc_strict_warning_flags().split():
        compile_command.append(item)
    compile_command.append('-o')
    compile_command.append(oname)
    proc = subprocess.Popen(compile_command,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
    stdout_data = proc.stdout.readlines()
    stderr_data = proc.stderr.readlines()
    if stdout_data<>[] or stderr_data<>[]:
        if stdout_data<>[]:
            for line in stdout_data:
                print line.rstrip()
        if stderr_data<>[]:
            for line in stderr_data:
                print line.rstrip()
        raise NameError('problem with compilation')

def simple_compile(fname, lname):

    import subprocess

    oname = source2lib(fname, lname)
    compile_command = ['g++','-c','-g',fname,'-O0','-o',oname]
    proc = subprocess.Popen(compile_command,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    stdout_data = proc.stdout.readlines()
    stderr_data = proc.stderr.readlines()
    if stdout_data<>[] or stderr_data<>[]:
        if stdout_data<>[]:
            for line in stdout_data:
                print line.rstrip()
        if stderr_data<>[]:
            for line in stderr_data:
                print line.rstrip()
        raise NameError('problem with compilation')    

def sensitive_compile(fname, lname):

    import os.path

    opath = fname.split('/')[-1].replace('cpp','o')
    if os.path.isfile(opath):
        os.remove(opath)

    if '/treecode/' in fname:
        simple_compile(fname, lname)
    elif 'RT.cpp' in fname:
        simple_compile(fname, lname)
    else:
        strict_compile(fname, lname)

def efficient_compile(fname, lname):

    import os
    import re

    oname = source2lib(fname,lname)
    if not os.path.isfile(oname):
        sensitive_compile(fname,lname)
    else:
        if os.path.getmtime(fname)>os.path.getmtime(oname):
            sensitive_compile(fname, lname)
        elif os.path.getmtime('./homebrew_makefile.py')>os.path.getmtime(oname):
            sensitive_compile(fname, lname)

        rel_dep_list = []
        for line in open(fname).readlines():
            if '#include' in line and not '<' in line:
                rel_dep_list.append(line)
        for dep_file in rel_dep_list:
            rel_path = dep_file.split('"')[-2]
            root_path = fname.replace(fname.split('/')[-1],'')
            header_path = os.path.normpath(os.path.join(root_path, rel_path))
            if os.path.getmtime(header_path)>os.path.getmtime(oname):
                sensitive_compile(fname, lname)
                break

def main():

    import os

    lname = 'library'
    safe_create(lname)

    # Erase old library file
    if os.path.isfile(lname+'/librich.a'):
        os.remove(lname+'/librich.a')

    for fname in get_source_files('source'):
        efficient_compile(fname,lname)

    # Archive
    os.system('ar cr ./'+lname+'/librich.a ./'+lname+'/*.o')

if __name__ == '__main__':

    main()



def l1_error_margin(a1,a2):

    abs_dif = [abs(x-y) for x,y in zip(a1,a2)]
    return sum(abs_dif)/len(abs_dif)

def consolidate_data(fname):

    import h5py
    import numpy

    f = h5py.File(fname)
    data = {'geometry':{},
            'hydrodynamic':{}}
    for field in data:
        for subfield in f[field]:
            data[field][subfield] = numpy.array(f[field][subfield])
    data['time'] = numpy.array(f['time'])
    return data

def main():

    graphic_flag = False

    if graphic_flag:
        #import matplotlib
        #matplotlib.use('Qt4Agg')
        import pylab
    
    import numpy
    import math
    import imp
    import h5py
    import os
    sedov_taylor = imp.load_source(
        'sedov_taylor',\
            os.environ['RICH_ROOT']+'/analytic/sedov_taylor.py')

    numeric = consolidate_data('final.h5')

    x_c = 0
    y_c = 0.5
    r_list = numpy.sqrt((numeric['geometry']['y_coordinate']-y_c)**2+
                        (numeric['geometry']['x_coordinate']-x_c)**2)
    p_front = numpy.max(numeric['hydrodynamic']['pressure'])
    r_front = r_list[numpy.argmax(numeric['hydrodynamic']['pressure'])]
    p_back = numeric['hydrodynamic']['pressure'][numpy.argmin(r_list)]

    r_relevant = [r for r in r_list if r<r_front]
    p_relevant = [numeric['hydrodynamic']['pressure'][i] for i in range(len(r_list)) 
                  if r_list[i]<r_front]

    # Analytic results
    g = 5./3.
    w = 0
    n = 3
    v_for_interp = numpy.linspace(1e-9+1./g,2./(g+1),num=1000)
    r_for_interp = [r_front*sedov_taylor.vtoz(v,w,g,n) 
                    for v in v_for_interp]
    p_for_interp = [p_back*sedov_taylor.vtop(v,w,g,n)/
                    sedov_taylor.vtop(1./g+1e-6,w,g,n)
                    for v in v_for_interp]

    l1 = l1_error_margin(p_relevant,
                         numpy.interp(r_relevant,
                                      r_for_interp,
                                      p_for_interp))

    f = open('gradesheet.txt','w')
    f.write(str(l1)+'\n')
    f.close()

    if graphic_flag:
        pylab.subplot(211)
        pylab.plot(r_relevant, p_relevant,'.')
        pylab.subplot(212)
        pylab.plot(r_for_interp,p_for_interp)
        pylab.show()

    return l1<23

if __name__=='__main__':

    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')




def main():

    import argparse
    import os

    parser = argparse.ArgumentParser()
    parser.add_argument('source_path',
                        help='path to file with main function')
    args = parser.parse_args()
    
    # Build
    hdf5_lib_path = os.environ['HDF5_LIB_PATH']
    rich_root = os.environ['RICH_ROOT']
    os.system('g++ -o ./rich.exe '+
              '-O3 '+
              args.source_path+' '+
              '-I '+rich_root+' '+
              (rich_root+'/library/*.o ').replace('//','/')+' '+
              (hdf5_lib_path+'/libhdf5_cpp.a').replace('//','/')+' '+
              (hdf5_lib_path+'/libhdf5_hl.a').replace('//','/')+' '+
              (hdf5_lib_path+'/libhdf5.a').replace('//','/')+' '+
              '-lpthread -lz -ldl ')

if __name__ == '__main__':

    main()



#! /usr/bin/python

def main():

    import numpy
    pa = 5.11232
    va = 1.53795
    rawd = numpy.loadtxt('res.txt')
    pn = rawd[0]
    vn = rawd[1]
    return abs(pa-pn)/(pa+pn)<0.01 and abs(va-vn)/(va+vn)<0.01

if __name__=='__main__':
    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')




def parse_delaunay(fname):

    import h5py
    import numpy

    res = {}
    with h5py.File(fname,'r+') as f:
        for field in f:
            res[field] = numpy.array(f[field])
    res['facets'] = numpy.reshape(res['triangles'],(-1,3))
    return res

def locate_point(x_list, y_list, xp, yp, tol = 1e-10):

    import numpy

    d_list = (x_list-xp)**2 + (y_list-yp)**2
    assert(numpy.min(d_list)<tol)
    return numpy.argmin(d_list)

def is_boxed(x, y, offset=0):

    return 1-offset>=x>=0+offset and 1-offset>=y>=0+offset

def main():

    import os
    #import pylab
    import numpy
    import re
    import glob

    if os.path.isfile('serial_ignore.txt'):
        return True

    whole = parse_delaunay('whole.h5')
    parts = [parse_delaunay(fname) for fname in 
             sorted(glob.glob('part_*.h5'),
                    key=
                    lambda x: int(re.search(r'part_(\d+).h5',x).group(1)))]

    whole['sorted facets'] = [sorted(itm) for itm in whole['facets']]
    for n, part in enumerate(parts):
        index_table = [
            locate_point(
                whole['x_coordinate'],
                whole['y_coordinate'],
                x,y) if is_boxed(x,y)
            else -1
            for x,y in 
            zip(part['x_coordinate'],part['y_coordinate'])]
        for facet in part['facets']:
            if numpy.all([itm<part['point number'] for itm in facet]):
                t_facet = sorted([index_table[itm] for itm in facet])
                if not t_facet in whole['sorted facets']:
                    return False
            elif numpy.all(
                [is_boxed(part['x_coordinate'][itm],
                          part['y_coordinate'][itm],0.04)
                 for itm in facet]) and numpy.any([itm<part['point number'] for itm in facet]):
                t_facet = sorted([index_table[itm] for itm in facet])
                if not t_facet in whole['sorted facets']:
                    return False                

    return True

if __name__ == '__main__':

    import os

    os.system('rm *.res;')
    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')



#! /usr/bin/python

def goodness_of_fit(a1, a2):

    import math

    diff2 = [(x-y)**2 for x,y in zip(a1,a2)]
    return math.sqrt(sum(diff2)/(max(a1)-min(a1))/len(a1))

def main():

    import numpy

    # Simulation results

    rawd = numpy.loadtxt('x_prof_initial.txt')
    xr0 = rawd[:,0]
    dr0 = rawd[:,1]
    pr0 = rawd[:,2]
    vr0 = rawd[:,3]
    cr0 = rawd[:,4]
    time = numpy.loadtxt('time.txt')

    xs0 = numpy.sort(xr0)
    ids0 = numpy.argsort(xr0)
    ds0 = [dr0[i] for i in ids0]
    ps0 = [pr0[i] for i in ids0]
    vs0 = [vr0[i] for i in ids0]
    cs0 = [cr0[i] for i in ids0]

    rawd = numpy.loadtxt('x_prof_final.txt')
    xr = rawd[:,0]
    dr = rawd[:,1]
    pr = rawd[:,2]
    vr = rawd[:,3]
    cr = rawd[:,4]

    xs = numpy.sort(xr)
    ids = numpy.argsort(xr)
    ds = [dr[i] for i in ids]
    ps = [pr[i] for i in ids]
    vs = [vr[i] for i in ids]
    cs = [cr[i] for i in ids]
    
    # Analytic results
    xa = [x+time*(c+v) for x,v,c in zip(xs0,vs0,cs0)]

    # Prepare for interpolation
    x_inside = [z for z in xa if z<numpy.max(xs) and 
                z>numpy.min(xs)]
    d_analytic = [ds0[i] for i in range(len(xa)) 
                  if xa[i]<numpy.max(xs) and
                  xa[i]>numpy.min(xs)]
    p_analytic = [ps0[i] for i in range(len(xa)) 
                  if xa[i]<numpy.max(xs) and
                  xa[i]>numpy.min(xs)]
    v_analytic = [vs0[i] for i in range(len(xa)) 
                  if xa[i]<numpy.max(xs) and
                  xa[i]>numpy.min(xs)]

    #d_analytic = numpy.interp(x_inside,xa,ds0)
    d_numeric = numpy.interp(x_inside,xs,ds)
    #p_analytic = numpy.interp(x_inside,xa,ps0)
    p_numeric = numpy.interp(x_inside,xs,ps)
    #v_analytic = numpy.interp(x_inside,xa,vs0)
    v_numeric = numpy.interp(x_inside,xs,vs)

    gof1 = goodness_of_fit(d_numeric,d_analytic)
    gof2 = goodness_of_fit(p_numeric,p_analytic)
    gof3 = goodness_of_fit(v_numeric,v_analytic)

    f = open('gradesheet.txt','w')
    f.write(str(gof1)+'\n')
    f.write(str(gof2)+'\n')
    f.write(str(gof3)+'\n')
    f.close()

    return gof1<0.2 and \
        gof2<0.61 and \
        gof3<0.3

import sys
if __name__=='__main__':
    print main()



def consolidate(fname,g=5./3.):

    import h5py
    import numpy

    f = h5py.File(fname)
    res = {}
    for field in f:
        res[field] = numpy.array(f[field])
    res['sound_speed'] = numpy.sqrt(g*res['pressure']/res['density'])
    res['abs_velocity'] = numpy.sqrt(res['x_velocity']**2+res['y_velocity']**2)
    res['mach_number'] = res['abs_velocity']/res['sound_speed']
    res['log_abs_velocity'] = numpy.log(1e-30+res['abs_velocity'])
    res['log_pressure'] = numpy.log(res['pressure'])
    res['log_density'] = numpy.log(res['density'])
    res['radius'] = numpy.sqrt(res['x_coordinate']**2+res['y_coordinate']**2)
    res['r_velocity'] = (res['x_velocity']*res['x_coordinate']+
                         res['y_velocity']*res['y_coordinate'])/res['radius']
    if 'entropy' in f:
        res['log_entropy'] = numpy.log(res['entropy'])
    f.close()
    return res

def h5_unite(f_list, g=5./3.):

    import numpy

    temp = [consolidate(fname) for fname in f_list]
    res = {}
    for field in temp[0]:
        res[field] = numpy.concatenate([itm[field] for itm in temp])

    return res

def get_user_args():

    from argparse import ArgumentParser

    parser = ArgumentParser(description='Displays a snapshot of a hydrodynamic simulation')
    #parser.add_argument('file_name',
    #                    help='path to output file')
    parser.add_argument('file_names', metavar='fn', nargs='+',
                        help='input files')
    parser.add_argument('field',
                        help='name of hydrodynamic variable')    
    return parser.parse_args()

def display_snapshot(fname,field):

    import matplotlib
    import numpy
    import pylab
    from matplotlib.collections import PolyCollection
    import matplotlib.pyplot as plt
    import matplotlib as mpl

    #raw = consolidate(fname)
    raw = h5_unite(fname)
    vert_idx_list = numpy.concatenate(([0],numpy.cumsum(raw['Number of vertices in cell'])))

    verts = []
    for i in range(len(raw[field])):
        lowbound = vert_idx_list[i]
        upbound = vert_idx_list[i+1]
        verts.append([[x,y] 
                      for x,y 
                      in zip(raw['x position of vertices'][lowbound:upbound],
                             raw['y position of vertices'][lowbound:upbound])])

    coll = PolyCollection(verts,array=raw[field],cmap=mpl.cm.jet,
                          edgecolors='none')
    fig, ax = plt.subplots()
    fig.suptitle(field+' @ t = '+str(raw['time'][0]))
    ax.add_collection(coll)
    ax.autoscale_view()
    ax.set_aspect('equal')

    fig.colorbar(coll, ax=ax)
    plt.show()

def main():

    args = get_user_args()

    display_snapshot(args.file_names, args.field)

if __name__ == '__main__':

    main()



#! /usr/bin/python

import matplotlib
matplotlib.use('Qt4Agg')
import pylab
import numpy
import sys

rfname = sys.argv[1]
vfname = sys.argv[2]

r = numpy.loadtxt(rfname)
v = numpy.loadtxt(vfname)

pylab.plot(r,v)
pylab.show()



def get_file_name_from_user():

    from argparse import ArgumentParser

    parser = ArgumentParser(description='replaces old style cast with new style cast')
    parser.add_argument('fname',help='file name')
    return parser.parse_args().fname

def main(fname):

    import re

    raw_data = open(fname).readlines()
    f = open(fname,'w')
    for line in raw_data:
        new_line = re.sub(r'([^a-zA-Z0-9_>])\(([a-zA-Z0-9_]+)\)([a-zA-Z0-9_.]+)\(\);',
                          r'\1static_cast<\2>(\3());',line)
        new_line = re.sub(r'([^a-zA-Z0-9_>])\(([a-zA-Z0-9_]+)\)([a-zA-Z0-9_.]+)\(\),',
                          r'\1static_cast<\2>(\3()),',line)
        new_line = re.sub(r'([^a-zA-Z0-9_>])\(([a-zA-Z0-9_]+)\)([a-zA-Z0-9_.]+)\(\)\)',
                          r'\1static_cast<\2>(\3()))',line)
        new_line = re.sub(r'([^a-zA-Z0-9_>])\(([a-zA-Z0-9_]+)\)([a-zA-Z0-9_.]+);',
                          r'\1static_cast<\2>(\3);',new_line)
        new_line = re.sub(r'([^a-zA-Z0-9_>])\(([a-zA-Z0-9_]+)\)([a-zA-Z0-9_.]+)\)',
                          r'\1static_cast<\2>(\3))',new_line)
        new_line = re.sub(r'([^a-zA-Z0-9_>])\(([a-zA-Z0-9_]+)\)([a-zA-Z0-9_.]+)\]',
                          r'\1static_cast<\2>(\3)]',new_line)
        new_line = re.sub(r'([^a-zA-Z0-9_>])\(([a-zA-Z0-9_]+)\)([a-zA-Z0-9_.]+)-([^>])',
                          r'\1static_cast<\2>(\3)-\4',new_line)
        f.write(new_line)
    f.close()        

if __name__ == '__main__':

    main(get_file_name_from_user())



#! /usr/bin/python

import math

def chi2_test(a1,a2):

    import math

    diff2 = [(x-y)**2 for x,y in zip(a1,a2)]
    return math.sqrt(sum(diff2)/len(a1))

def L1_test(a1,a2):

    import math

    abs_diff = [abs(x-y) for x,y in zip(a1,a2)]
    return sum(abs_diff)/len(a1)

def goodness_of_fit(a1, a2,test_name='L1'):

    import math

    if 'chi2'==test_name:
        return chi2_test(a1,a2)
    elif 'L1'==test_name:
        return L1_test(a1,a2)
    else:
        raise NameError("I don't know test "+
                        test_name)

def density_init_prof(x):

    if x<=0 or x>=0.5:
        return 2
    else:
        return 1

def density_prof(x,t):

    xeq = x-t
    if xeq < 0:
        xeq = xeq + 1
    return density_init_prof(xeq)

def pressure_prof(x):

    return 1

def velocity_prof(x):

    return 1

def main():

    import os

    import numpy

    time = numpy.loadtxt('time.txt')
    x_list = numpy.loadtxt('cell_centres.txt')
    d_list = numpy.loadtxt('densities.txt')
    p_list = numpy.loadtxt('pressures.txt')
    v_list = numpy.loadtxt('velocities.txt')
    da = [density_prof(x,time) for x in x_list]
    pa = [pressure_prof(x) for x in x_list]
    va = [velocity_prof(x) for x in x_list]

    if False:
        import pylab

        pylab.subplot(311)
        pylab.plot(x_list,d_list,x_list,da)
        pylab.ylabel('Density')

        pylab.subplot(312)
        pylab.plot(x_list,p_list,x_list,pa)
        pylab.ylabel('Pressure')

        pylab.subplot(313)
        pylab.plot(x_list,v_list,x_list,va)
        pylab.xlabel('Distance')
        pylab.ylabel('Velocity')

        pylab.show()

    test_name = 'L1'
    gof1 = goodness_of_fit(d_list,da,test_name)
    gof2 = goodness_of_fit(p_list,pa,test_name)
    gof3 = goodness_of_fit(v_list,va,test_name)

    f = open('gradesheet.txt','w')
    f.write(str(gof1)+'\n')
    f.write(str(gof2)+'\n')
    f.write(str(gof3)+'\n')
    f.close()

    return gof1<0.06 and gof2<0.02 and gof3 < 0.04

import sys

if __name__=='__main__':

    print main()



#!/usr/bin/env python
# -*- Coding: UTF-8 -*-

# ---------------------------------------------------------------------------
# Open Asset Import Library (ASSIMP)
# ---------------------------------------------------------------------------
#
# Copyright (c) 2006-2020, ASSIMP Development Team
#
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, 
# with or without modification, are permitted provided that the following 
# conditions are met:
# 
# * Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
# 
# * Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
# 
# * Neither the name of the ASSIMP team, nor the names of its
#   contributors may be used to endorse or promote products
#   derived from this software without specific prior
#   written permission of the ASSIMP Development Team.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# ---------------------------------------------------------------------------

"""Update PyAssimp's texture type constants C/C++ headers.

This script is meant to be executed in the source tree, directly from
port/PyAssimp/gen
"""

import os
import re

REenumTextureType = re.compile(r''
                r'enum\saiTextureType'    # enum aiTextureType
                r'[^{]*?\{'               # {
                r'(?P<code>.*?)'          #   code 
                r'\};'                    # };
                , re.IGNORECASE + re.DOTALL + re.MULTILINE)

# Replace comments
RErpcom = re.compile(r''
                r'\s*(/\*+\s|\*+/|\B\*\s?|///?!?)'   # /**
                r'(?P<line>.*?)'                     #  * line 
                , re.IGNORECASE + re.DOTALL)

# Remove trailing commas
RErmtrailcom = re.compile(r',$', re.IGNORECASE + re.DOTALL)

# Remove #ifdef __cplusplus   
RErmifdef = re.compile(r''
                r'#ifndef SWIG'                       # #ifndef SWIG
                r'(?P<code>.*)'                       #   code 
                r'#endif(\s*//\s*!?\s*SWIG)*'  # #endif
                , re.IGNORECASE + re.DOTALL)

path = '../../../include/assimp'

files = os.listdir (path)
enumText = ''
for fileName in files:
    if fileName.endswith('.h'):
      text = open(os.path.join(path, fileName)).read()
      for enum in REenumTextureType.findall(text):
        enumText = enum

text = ''
for line in enumText.split('\n'):
  line = line.lstrip().rstrip()
  line = RErmtrailcom.sub('', line)
  text += RErpcom.sub('# \g<line>', line) + '\n'
text = RErmifdef.sub('', text)

file = open('material.py', 'w')
file.write(text)
file.close()

print("Generation done. You can now review the file 'material.py' and merge it.")



def collect_data(folder_list):

    unsorted_lists = {'resolution':[],
                      'density l1':[],
                      'pressure l1':[],
                      'velocity l1':[]}

    import numpy
    for folder in folder_list:
        res = int(numpy.loadtxt(folder+'/resolution.txt'))
        rawd = numpy.loadtxt(folder+'/gradesheet.txt')
        unsorted_lists['resolution'].append(res)
        unsorted_lists['density l1'].append(rawd[0])
        unsorted_lists['pressure l1'].append(rawd[1])
        unsorted_lists['velocity l1'].append(rawd[2])

    mask = sorted(range(len(unsorted_lists['resolution'])),
                  key=lambda i:unsorted_lists['resolution'][i])
    sorted_lists = {}
    for field in unsorted_lists:
        sorted_lists[field] = [unsorted_lists[field][i] for i in mask]
    
    return sorted_lists

if __name__=='__main__':
    
    import sys

    folder_list = sys.argv[1:-1]
    output_file = sys.argv[-1]
    if not ".txt" in output_file:
        raise NameError("Expected .txt extension in output file. "+
                        "Was an output file provided?")
    temp = collect_data(folder_list)
    
    f = open(output_file,'w')
    for i in range(len(temp['resolution'])):
        for field in ['resolution','density l1','pressure l1','velocity l1']:
            f.write(str(temp[field][i])+' ')
        f.write('\n')
    f.close()



#! /usr/bin/python

import matplotlib
matplotlib.use('Qt4Agg')
import pylab
import numpy
import sys
import math

class Vector2D:
    """
    A mathematical vector in 2d
    """

    def __init__(self, x, y):
        """
        Class constructor
        Input:
        x - x Component
        y - y Component
        """

        self._x = x
        self._y = y
        return

    def SetX(x):
        """
        Sets the x component of a vector
        Input:
        x - x Component
        """

        self._x = x
        return

    def SetY(y):
        """
        Sets the y component of a vector
        Input:
        y - y Component
        """

        self._y = y
        return
    
    def GetX(self):
        """
        Returns the x component of a vector
        """

        return self._x

    def GetY(self):
        """
        Returns the y component of a vector
        """

        return self._y

    def __add__(self, v):
        """
        Adds the components of one vector to another
        """

        return Vector2D(self.GetX()+v.GetX(), \
                            self.GetY()+v.GetY())

    def __sub__(self, v):
        """
        Subtracting the components of one vector from another
        """

        return Vector2D(self.GetX()-v.GetX(), \
                            self.GetY()-v.GetY())

    def __mul__(self, c):
        """
        Multiplying each vector term by a scalar
        """

        return Vector2D(self.GetX()*c, self.GetY()*c)

    def __div__(self, c):
        """
        Dividing each vector term by a scalar
        """

        return Vector2D(self.GetX()/c, self.GetY()/c)

def crossz(v):
    
    return Vector2D(v.GetY(), -v.GetX())

class Edge:
    """
    Interface between two cells
    """

    def __init__(self, vertices, neighbors):
        """
        Class constructor
        Input:
        vertices - List of vertices
        neighbors - List of the indices of neighbor cells
        """
        self._vertices = vertices
        self._neighbors = neighbors
        return

    def GetVertices(self):
        """
        Returns a list of vertices
        """

        return self._vertices

    def GetNeighbors(self):
        """
        Returns a list of indices of neibor cells
        """

        return self._neighbors

def ScalarProd(v1, v2):
    
    return v1.GetX()*v2.GetX()+ \
        v1.GetY()*v2.GetY()

def Modulus(v):

    return ScalarProd(v,v)**0.5

def SubtendedAngle(v1, v2):

    aux = ScalarProd(v1,v2)/(Modulus(v1)*Modulus(v2))
    if(aux>1):
        aux = 1
    if(aux<-1):
        aux = -1
    return math.acos(aux)

def CalcCentroid(edge):
    
    vertices = edge.GetVertices()
    return (vertices[0]+vertices[1])*0.5

def CalcCenter(edges):

    center = Vector2D(0,0)
    for i in edges:
        center = center + CalcCentroid(i)/len(edges)
    return center

def  sgn(x):

    if(x>0):
        return 1
    if(x<0):
        return -1
    else:
        return 0

def DirAngle(v, vref):

    return SubtendedAngle(v, vref)*sgn(ScalarProd(v,crossz(vref)))

def IsRightTriangle(p1, p2, p3):

    return ScalarProd(p3-p1,crossz(p2-p1))>=0

class Polygon:
    """
    Convex polygon
    """

    def __init__(self, edges):
        """
        Class constructor
        Input:
        edges - Unordered list of vertices
        """
        
        self._center = CalcCenter(edges)
        self._vertices = []
        for i in edges:
            edge_vertices = i.GetVertices()
            if IsRightTriangle(self._center, edge_vertices[0], edge_vertices[1]):
                self._vertices.append(edge_vertices[0])
            else:
                self._vertices.append(edge_vertices[1])
        for i in range(len(self._vertices)):
            for j in range(i+1,len(self._vertices)):
                for k in range(j+1,len(self._vertices)):
                    if ~IsRightTriangle(self._vertices[i], \
                                            self._vertices[j], \
                                            self._vertices[k]):
                        self._vertices[j], self._vertices[k] = self._vertices[k], self._vertices[j]
    
    def GetVertices(self):
        """
        Returns a list of vertices, in a clockwise order
        """

        return self._vertices

    def GetCenter(self):
        """
        Returns the center of the polygon
        """

        return self._center



# Code starts here

rawd = numpy.loadtxt(sys.argv[1])
z = numpy.loadtxt(sys.argv[2])
zeps = 1e-15
zn = (z-z.min())/(z.max()-z.min()+zeps)

# Create edges
edges = []
for i in rawd:
    v1 = Vector2D(i[0],i[1])
    v2 = Vector2D(i[3],i[4])
    i1 = int(i[2])
    i2 = int(i[5])
    e0 = Edge([v1,v2],[i1,i2])
    edges.append(e0)

# Sort edges by polygon
upe = [] # unordered polygon edges
for i in range(len(z)):
    upe.append([])
for i in edges:
    for j in i.GetNeighbors():
        if j>-1:
            upe[j].append(i)

counter = 0
for i,j in zip(upe, zn):

    aux = Polygon(i)
    vert_x_list = [k.GetX() for k in aux.GetVertices()]
    vert_y_list = [k.GetY() for k in aux.GetVertices()]
    pylab.fill(vert_x_list, vert_y_list,
               fc=matplotlib.cm.jet(j))

pylab.axis('equal')

ax, _ = matplotlib.colorbar.make_axes(pylab.gca())
matplotlib.colorbar.ColorbarBase(ax,norm=matplotlib.colors.Normalize(vmin=min(z),vmax=max(z)))

pylab.show()



class Primitive:
    """
    Primitive hydrodynamic variables
    """

    def __init__(self,density,pressure,xvelocity,
                 yvelocity,energy,soundspeed):

        self.density = density
        self.pressure = pressure
        self.xvelocity = xvelocity
        self.yvelocity = yvelocity
        self.energy = energy
        self.soundspeed = soundspeed

class Flux:
    """
    Hydrodynamic flux
    """

    def __init__(self,mass,xmomentum,ymomentum,energy):

        self.mass = mass
        self.xmomentum = xmomentum
        self.ymomentum = ymomentum
        self.energy = energy

def read_primitive(fname):

    import numpy

    rawd = numpy.loadtxt(fname)
    return Primitive(rawd[0],
                     rawd[1],
                     rawd[4],
                     rawd[5],
                     rawd[2],
                     rawd[3])

def read_flux(fname):

    import numpy

    rawd = numpy.loadtxt(fname)
    return Flux(rawd[0],
                rawd[1],
                rawd[2],
                rawd[3])

def approx_compare(val1,val2,thres):

    if 0==val1 and 0==val2:
        return True
    else:
        return abs(val1-val2)/abs(val1+val2)<thres

def main():

    left = read_primitive('left.txt')
    right = read_primitive('right.txt')
    numeric_flux = read_flux('flux.txt')

    exact_mass_flux = left.xvelocity*left.density
    exact_xmomentum_flux = left.pressure+\
        left.density*(left.xvelocity)**2
    exact_ymomentum_flux = left.density*\
        left.xvelocity*left.yvelocity
    exact_energy_flux = left.xvelocity*\
        (left.pressure+\
             left.density*left.energy+\
             0.5*left.density*left.xvelocity**2)

    f = open('gradesheet.txt','w')
    f.write('Component Numeric Exact\n')
    f.write('Mass '+str(numeric_flux.mass)+' '+str(exact_mass_flux)+'\n')
    f.write('xMomentum '+str(numeric_flux.xmomentum)+' '+str(exact_xmomentum_flux)+'\n')
    f.write('yMomentum '+str(numeric_flux.ymomentum)+' '+str(exact_ymomentum_flux)+'\n')
    f.write('Energy '+str(numeric_flux.energy)+' '+str(exact_energy_flux)+'\n')

    flag1 = approx_compare(numeric_flux.mass,exact_mass_flux,0.01)
    flag2 = approx_compare(numeric_flux.xmomentum,exact_xmomentum_flux,0.01)
    flag3 = approx_compare(numeric_flux.ymomentum,exact_ymomentum_flux,0.01)
    flag4 =  approx_compare(numeric_flux.energy,exact_energy_flux,0.01)
    return flag1 and flag2 and flag3 and flag4

if __name__=='__main__':

    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')



#! /usr/bin/python

import matplotlib
matplotlib.use('Qt4Agg')
import pylab
import numpy
import sys
import os
import imp
enrs = imp.load_source('enrs','../analytic/enrs.py')

pref = sys.argv[0].replace('draw_prof_1d.py','')

rawd = numpy.loadtxt(pref+'prof1d.txt')
x = rawd[:,0]
d = rawd[:,1]
p = rawd[:,2]
v = rawd[:,3]

xs = numpy.sort(x)
ids = numpy.argsort(x)
ds = [d[i] for i in ids]
ps = [p[i] for i in ids]
vs = [v[i] for i in ids]

left = enrs.Primitive(1,2,0)
right = enrs.Primitive(1,1,0)
prof = enrs.RiemannProfile(left,right,5./3.)
t = numpy.loadtxt(pref+'time.txt')
offset = 0.5;
da = [prof.CalcPrim((i-offset)/t).Density for i in xs]
pa = [prof.CalcPrim((i-offset)/t).Pressure for i in xs]
va = [prof.CalcPrim((i-offset)/t).Velocity for i in xs]

pylab.subplot(311)
pylab.plot(xs,ds,xs,da)
pylab.xlabel('x')
pylab.ylabel('Density')
pylab.title('t = '+str(t))

pylab.subplot(312)
pylab.plot(xs,ps,xs,pa)
pylab.xlabel('x')
pylab.ylabel('Pressure')

pylab.subplot(313)
pylab.plot(xs,vs,xs,va)
pylab.xlabel('x')
pylab.ylabel('Velocity')

pylab.show()



def main():

    import numpy
    import math
    import h5py
    import glob
	
    with h5py.File('final.h5','r+') as f:
        density_list = numpy.array(f['hydrodynamic']['density'])
        chi_2 = numpy.sqrt(sum((density_list-1)**2))/len(density_list)
    mass = numpy.loadtxt('mass.txt')
    return chi_2<1e-8 and abs(mass-4)<3e-8

 
if __name__ == '__main__':

    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')




def l1_fit(a1, a2):

    return sum([abs(x-y) for x,y in zip(a1,a2)])/len(a1)

def pressure_dist(r):

    import math

    if r<0.2:
        return 5+(25./2.)*r**2
    elif r>0.4:
        return 3+4*math.log(2.)
    else:
        return 9+(25./2.)*r**2-20*r+4*math.log(r/0.2)

def azimuthal_velocity_dist(r):

    import math

    if r<0.2:
        return 5*r
    elif r>0.4:
        return 0
    else:
        return 2-5*r

def main():

    import numpy
    import math
    import h5py
    import glob

    np = len(glob.glob('process_*_final.h5'))
    if np>0:
        rx_list = []
        ry_list = []
        d_list = []
        p_list = []
        vx_list = []
        vy_list = []
        for fname in glob.glob('process_*_final.h5'):
            f = h5py.File(fname)
            rx_list.extend(f['x_coordinate'])
            ry_list.extend(f['y_coordinate'])
            d_list.extend(f['density'])
            p_list.extend(f['pressure'])
            vx_list.extend(f['x_velocity'])
            vy_list.extend(f['y_velocity'])
    else:
        h5f = h5py.File('final.h5')
        rx_list = h5f['geometry']['x_coordinate']
        ry_list = h5f['geometry']['y_coordinate']
        d_list = h5f['hydrodynamic']['density']
        p_list = h5f['hydrodynamic']['pressure']
        vx_list = h5f['hydrodynamic']['x_velocity']
        vy_list = h5f['hydrodynamic']['y_velocity']

    r_list = [math.sqrt(x**2+y**2)
              for x,y in zip(rx_list,ry_list)]
    r_sorted = numpy.sort(r_list)
    vf_list = [(vy*x-vx*y)/math.sqrt(x**2+y**2)
               for x,y,vx,vy in
               zip(rx_list,
                   ry_list,
                   vx_list,
                   vy_list)] 

    vr_list = [(vy*y+vx*x)/math.sqrt(x**2+y**2)
               for x,y,vx,vy in
               zip(rx_list,
                   ry_list,
                   vx_list,
                   vy_list)] 

    if False:
        import pylab
        pylab.subplot(221)
        pylab.plot(r_list,p_list,'.')
        pylab.plot(r_sorted,[pressure_dist(r) for r in r_sorted])

        pylab.subplot(222)
        pylab.plot(r_list,vf_list,'.')
        pylab.plot(r_sorted,
                   [azimuthal_velocity_dist(r)
                    for r in r_sorted])

        pylab.subplot(223)
        pylab.plot(r_list,vr_list,'.')
        pylab.plot(r_sorted,
                   0*r_sorted)

        pylab.subplot(224)
        pylab.plot(r_list,d_list,'.')
        pylab.plot(r_sorted,
                   [1 for r in r_sorted])

        pylab.show()

    l1_density = l1_fit(d_list, [1 for r in r_list])
    l1_pressure = l1_fit(p_list,
                         [pressure_dist(r) for r in r_list])
    l1_radial_velocity = l1_fit(
        vr_list,[0 for r in r_list])
    l1_azimuthal_velocity = l1_fit(
        vf_list,[azimuthal_velocity_dist(r) for r in r_list])

    f = open('gradesheet.txt','w')
    f.write(str(l1_density)+'\n')
    f.write(str(l1_pressure)+'\n')
    f.write(str(l1_radial_velocity)+'\n')
    f.write(str(l1_azimuthal_velocity)+'\n')
    f.close()

    return l1_density<3.3e-4 and \
        l1_pressure<2.9e-3 and \
        l1_radial_velocity<1.3e-3 and \
        l1_azimuthal_velocity<2.1e-3

if __name__=='__main__':
    
    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')




def L1_error_norm(a1, a2):

    abs_diff = [abs(x-y) for x,y in zip(a1,a2)]
    return sum(abs_diff)/len(a1)

def chi2_error_norm(a1,a2):

    import math

    diff2 = [(x-y)**2 for x,y in zip(a1,a2)]
    return math.sqrt(sum(diff2)/len(a1))

def error_norm(a1, a2, test_name='L1'):

    if 'L1'==test_name:
        return L1_error_norm(a1,a2)
    elif 'chi2'==test_name:
        return chi2_error_norm(a1,a2)
    else:
        raise NameError('Unknown test name '+test_name)

def calc_sound_speed(d,p,g):

    import math

    return math.sqrt(g*p/d)

def simple_waves_propagate(x_list,
                           d_list,
                           p_list,
                           v_list,
                           time, g,
                           direction):

    if 0==direction or 1==direction or -1==direction:
        c_list = [calc_sound_speed(d,p,g) for d,p in zip(d_list,p_list)]
        propag_vel = [v+direction*c for v,c in zip(v_list,c_list)]
        res = [x+time*u for x,u in zip(x_list,propag_vel)]
        return res
    else:
        raise NameError('direction must be either 0,1 or -1')

def main():

    import numpy
    import h5py

    initial_data = h5py.File('initial.h5')
    x_init = initial_data['grid']
    d_init = initial_data['density']
    p_init = initial_data['pressure']
    v_init = initial_data['x_velocity']

    final_data = h5py.File('final.h5')
    time = numpy.array(final_data['time'])[0]
    x_numeric = final_data['grid']
    d_numeric = final_data['density']
    p_numeric = final_data['pressure']
    v_numeric = final_data['x_velocity']

    x_exact = simple_waves_propagate(x_init,
                                     d_init,
                                     p_init,
                                     v_init,
                                     time, 5./3.,1)

    if False:
        
        import pylab

        pylab.subplot(311)
        pylab.plot(x_numeric, d_numeric)
        pylab.plot(x_exact, d_init)
        pylab.plot(x_init,d_init)

        pylab.subplot(312)
        pylab.plot(x_numeric,p_numeric)
        pylab.plot(x_exact,p_init)
        pylab.plot(x_init,p_init)

        pylab.subplot(313)
        pylab.plot(x_numeric,v_numeric)
        pylab.plot(x_exact,v_init)
        pylab.plot(x_init,v_init)

        pylab.show()

    test_name = 'L1'
    xc = [x for x in x_exact if x>min(x_numeric) and x<max(x_numeric)]
    d1 = numpy.interp(xc,x_numeric,d_numeric)
    d2 = numpy.interp(xc,x_exact,d_init)
    errnorm1 = error_norm(d1,d2,test_name)

    p1 = numpy.interp(xc,x_numeric,p_numeric)
    p2 = numpy.interp(xc,x_exact,p_init)
    errnorm2 = error_norm(p1,p2,test_name)

    v1 = numpy.interp(xc,x_numeric,v_numeric)
    v2 = numpy.interp(xc,x_exact,v_init)
    errnorm3 = error_norm(v1,v2,test_name)

    f = open('gradesheet.txt','w')
    f.write(str(errnorm1)+'\n')
    f.write(str(errnorm2)+'\n')
    f.write(str(errnorm3)+'\n')
    f.close()

    return errnorm1<0.3 and errnorm2 < 1.7 and errnorm3 < 0.22
                                     
if __name__=='__main__':
    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')




"""
Based on
http://www.grc.nasa.gov/WWW/k-12/airplane/oblique.html
"""

def arcsec(x):

    import math

    return math.acos(1/x)

def arccot(x):

    import math

    return math.atan(1/x)

def critical_shock_angle(m,g):

    """
    Calculates the shock angle at which the transition from weak to strong shock occurs
    Input:
    m - Mach number
    g - Adiabatic index
    """

    import math

    return arcsec(2*m*math.sqrt(-(g/(-4 + (1 - 3*g)*m**2 + math.sqrt((1 + g)*(16 + \
                                                                                  8*(-1 + g)*m**2 + (1 + g)*m**4))))))

def calc_deflection_angle(m,g,s):

    """
    Calculates the deflection angle
    Input:
    m - Mach number
    g - Adiabatic index
    s - Shock angle
    """

    import math
    if math.sin(s)==1./m:
        return 0
    cota = math.tan(s)*(0.5*(g+1)*m**2/(m**2*math.sin(s)**2-1)-1)
    return arccot(cota)

def critical_deflection_angle(m,g):

    """
    Calculates the critical deflection angle
    i.e. the angle at the transition from weak to
    strong shock wave
    Input:
    m - Mach number
    g - Adiabatic index
    """

    sc = critical_shock_angle(m,g)
    return calc_deflection_angle(m,g,sc)

def minimum_shock_angle(m):

    """
    Calculates the shock angle
    for which the deflection angle is zero
    Input:
    m - Mach number
    """

    import math

    return math.asin(1/float(m))

class ShockAngleEqn:

    def __init__(self,m,g,a):

        """
        Class constructor
        Input:
        m - Mach number
        g - Adiabatic index
        a - Deflection angle
        """

        self.m = m
        self.g = g
        self.a = a

    def eval(self, s):

        return self.a - \
            calc_deflection_angle(self.m,
                                  self.g,s)

def bisection(f,xl,xh,n):

    fl = f.eval(xl)
    fh = f.eval(xh)
    
    if fl*fh>0:
        raise NameError('Root is not bracketed')
    if fl==0:
        return xl
    if fh==0:
        return xh

    for i in range(n):
        xm = 0.5*(xh+xl);
        fm = f.eval(xm)
        if fm==0:
            return xm
        
        if fl*fm>0:
            fl = fm
            xl = xm
        elif fh*fm:
            fh = fm
            xh = xm
        else:
            raise NameError('Function is not monotonous')

    return xm

def calc_shock_angle(m,g,a):

    eqn = ShockAngleEqn(m,g,a)
    sh = critical_shock_angle(m,g)
    sl = minimum_shock_angle(m)
    return bisection(eqn,sh,sl,20)



#! /usr/bin/python

def make_temp_folder(pref,uname):

    import re

    res = pref+'_'+uname
    res = res.replace('.','')
    res = res.replace('/','_')
    res = res.replace('__','_')
    res = re.sub(r'_$','',res)
    return res

def safe_remove_folder(fpath):
    
    import os
    os.system('touch '+fpath)
    os.system('rm -rf '+fpath)

class Tester:
    """
    A class that manages automatic tests
    """

    def __init__(self, tfname, srcdir):
        """
        Class constructor
        Input:
        tfname - Name of temporary folder
        scrdir - Path to source
        """

        self.__tfname = tfname
        self.__srcdir = srcdir
        return

    def DelTempDirs(self):
        """
        Deletes the temporary folder and all files within
        """

        import os
        os.system('rm -rf '+self.__tfname+'*')
        return

    def RunSingleTest(self, dname):
        """
        Runs a single test
        Input:
        dname - Path to directory
        """

        import os
        import imp

        temp_folder = make_temp_folder(self.__tfname,dname)
        safe_remove_folder(temp_folder)
        os.mkdir(temp_folder)
        os.system('cp -r ../'+self.__srcdir+' '+temp_folder)
        os.system('cp '+dname+'/* '+temp_folder)
        os.system('cp ./automaker.py '+temp_folder)
        myhome = os.getcwd()
        os.chdir(temp_folder)
        automaker = imp.load_source('automaker',os.getcwd()+'/automaker.py')
        intrp = automaker.CppInterpreter(\
            'g++','-Wfatal-errors -Wall -Wextra -pedantic -Werror -std=c++0x -O3')
        os.system('g++ -c source/treecode/*.cpp')
        intrp.Interpret('test.cpp')
        mod = imp.load_source('mod',os.getcwd()+'/test.py')
        res = mod.main()
        os.chdir(myhome)
        return res

def ListTestFolders(dname):
    """
    Returns a list of folders with test files
    Input:
    dname - Root folder
    """

    import os
    res  = []
    for roots, dirs, files in os.walk(dname):
        if roots.find('.svn')==-1 and len(roots)>1:
            if os.path.isfile(roots+'/test.py'):
                res.append(roots)
    return res

import sys
import os

tst = Tester('temp','source')

if len(sys.argv)==1:
    # Run all tests
    tst.DelTempDirs()
    ll = ListTestFolders('.')
    myhome = os.getcwd()
    for i in ll:
        try:
            tf = tst.RunSingleTest(i)
            if tf:
                print i+' ... passed'
            else:
                print i+' ... failed'
        except Exception, err:
            print i+' ... failed'
            print err
        os.chdir(myhome)
        tst.DelTempDirs()
elif len(sys.argv)==2:
    # Run a single test
    ll = sys.argv[1]
    try:
        tf = tst.RunSingleTest(ll)
        if tf:
            print ll+' ... passed'
        else:
            print ll+' ... failed'
    except Exception, err:
        print ll+' ... failed'
        print err
else:
    print 'unknown opion'
    



#! /usr/bin/python

def my_sign(x):

    if x>0:
        return 1.0
    elif x<0:
        return -1.0
    elif x==0:
        return 0.0
    else:
        raise NameError('Error in my_sign: x is not numeric')

def afd_single_mode(x,t,k,omega,v,g0,dx,xi):

    from cmath import exp

    numerator = dx*exp(1j*k*x)*g0*xi*(exp(1j*omega*t) - (1 + (-1 + \
exp(-(1j*dx*k*my_sign(v))))*xi)**((t*abs(v))/(dx*xi)))

    denominator = (-1 + exp((1j*dx*omega*xi)/abs(v)) + xi - \
xi/exp(1j*dx*k*my_sign(v)))*abs(v)

    return numerator/denominator

def afd_standing_wave(x,t,k,omega,v,g0,dx,xi):

    return 0.25*(afd_single_mode(x,t,k,omega,v,g0,dx,xi)-
                 afd_single_mode(x,t,-k,omega,v,g0,dx,xi)-
                 afd_single_mode(x,t,k,-omega,v,g0,dx,xi)+
                 afd_single_mode(x,t,-k,-omega,v,g0,dx,xi))

def afd_standing_wave_2(x,t,k,omega,v,g0,dx,xi):

    import cmath

    return (dx*g0*xi*(-((cmath.exp(2*1j*k*x)*(cmath.exp(-(1j*omega*t)) - (1 + \
(-1 + cmath.exp(-(1j*dx*k)))*xi)**((t*v)/(dx*xi))))/(-1 + \
cmath.exp(-(1j*dx*omega*xi)/v) + xi - xi/cmath.exp(1j*dx*k))) + \
(cmath.exp(2*1j*k*x)*(cmath.exp(1j*omega*t) - (1 + (-1 + \
cmath.exp(-(1j*dx*k)))*xi)**((t*v)/(dx*xi))))/(-1 + \
cmath.exp((1j*dx*omega*xi)/v) + xi - xi/cmath.exp(1j*dx*k)) + \
(cmath.exp(-(1j*omega*t)) - (1 + (-1 + \
cmath.exp(1j*dx*k))*xi)**((t*v)/(dx*xi)))/(-1 + \
cmath.exp(-(1j*dx*omega*xi)/v) + xi - cmath.exp(1j*dx*k)*xi) + \
(-cmath.exp(1j*omega*t) + (1 + (-1 + \
cmath.exp(1j*dx*k))*xi)**((t*v)/(dx*xi)))/(-1 + \
cmath.exp((1j*dx*omega*xi)/v) + xi - \
cmath.exp(1j*dx*k)*xi)))/(4.*cmath.exp(1j*k*x)*v)

def afd_hydro_profiles(x_list,t,k,omega,g0,dx,xi,rho0,p0,gamma):

    import math

    c0 = math.sqrt(gamma*p0/rho0)
    j_p_list = [afd_standing_wave(x,t,k,omega,c0,g0,dx,xi).real for x in x_list]
    j_m_list = [afd_standing_wave(x,t,k,omega,-c0,g0,dx,xi).real for x in x_list]
    v_list = [0.5*(j_p+j_m) for j_p, j_m in zip(j_p_list,j_m_list)]
    rho_list = [rho0+0.5*(j_p-j_m)*rho0/c0
                for j_p, j_m
                in zip(j_p_list, j_m_list)]
    p_list = [p0+gamma*p0*(rho-rho0)/rho0 for rho in rho_list]

    return rho_list, p_list, v_list

def l1_fit_factor(a1, a2):

    import math

    diff2 = [abs(x-y) for x,y in zip(a1,a2)]
    return sum(diff2)/len(a1)

def main():

    import numpy
    import os
    #import imp
    import math

    x_list, d_list, p_list, vx_list, vy_list = \
        numpy.loadtxt('prof1d.txt', unpack=True)

    rho_0 = numpy.loadtxt('mean_density.txt')
    amp = numpy.loadtxt('amplitude.txt')
    k = 2*numpy.pi/numpy.loadtxt('wavelength.txt')
    vp = numpy.loadtxt('phase_velocity.txt')
    p_0 = numpy.loadtxt('mean_pressure.txt')
    gamma = numpy.loadtxt('adiabatic_index.txt')
    c_0 = numpy.sqrt(gamma*p_0/rho_0)
    t = numpy.loadtxt('time.txt')
    
    s_list = [p/d**gamma for d,p in zip(d_list,p_list)]
    j_p_list = [v+(rho-rho_0)*c_0/rho_0
                for rho,v in zip(d_list,vx_list)]
    j_m_list = [v-(rho-rho_0)*c_0/rho_0
                for rho,v in zip(d_list,vx_list)]

    def density_exact(x):

        return rho_0-\
            amp*rho_0*math.cos(k*x)*\
            (vp*math.sin(c_0*k*t)-c_0*math.sin(vp*k*t))/\
            (k*c_0*(c_0-vp)*(c_0+vp))

    def pressure_exact(x):

        return p_0-\
            amp*c_0*rho_0*math.cos(k*x)*\
            (vp*math.sin(c_0*k*t)-c_0*math.sin(k*t*vp))/\
            (k*(c_0-vp)*(c_0+vp))

    def velocity_exact(x):

        return -amp*vp*math.sin(k*x)*\
            (math.cos(k*t*vp)-math.cos(k*t*c_0))/\
            (k*(c_0-vp)*(c_0+vp))

    d_exact = [density_exact(x)
               for x in x_list]
    p_exact = [pressure_exact(x)
               for x in x_list]
    v_exact = [velocity_exact(x)
               for x in x_list]
    s_exact = [p/d**gamma for d,p in zip(d_exact,p_exact)]
    j_p_exact = [v+(rho-rho_0)*c_0/rho_0
                 for rho,v in zip(d_exact,v_exact)]
    j_m_exact = [v-(rho-rho_0)*c_0/rho_0
                 for rho,v in zip(d_exact,v_exact)]

    dx = 1.0/math.sqrt(len(x_list))
    d_afd, p_afd, v_afd =  afd_hydro_profiles(x_list,t,k,k*vp,amp,dx,0.3,rho_0,p_0,gamma)
    s_afd = [p/d**gamma for d,p in zip(d_afd,p_afd)]
    j_p_afd = [v+(d-rho_0)*c_0/rho_0
               for d,v in zip(d_afd,v_afd)]
    j_m_afd = [v-(d-rho_0)*c_0/rho_0
               for d,v in zip(d_afd,v_afd)]

    if False:

        import pylab
        #pylab.subplot(311)
        #pylab.plot(x_list, s_list, '.')
        #pylab.plot(x_list, s_exact, '.')
        #pylab.plot(x_list, s_afd, '.')
        #pylab.subplot(312)
        #pylab.plot(x_list, j_p_list, '.')
        #pylab.plot(x_list, j_p_exact, '.')
        #pylab.plot(x_list, j_p_afd, '.')
        #pylab.subplot(313)
        #pylab.plot(x_list, j_m_list, '.')
        #pylab.plot(x_list, j_m_exact, '.')
        #pylab.plot(x_list, j_m_afd, '.')
        #pylab.show()

        pylab.subplot(311)
        pylab.plot(x_list, d_list, '.')
        pylab.plot(x_list, d_exact, '.')
        pylab.plot(x_list, d_afd, '.')
        pylab.subplot(312)
        pylab.plot(x_list, p_list, '.')
        pylab.plot(x_list, p_exact, '.')
        pylab.plot(x_list, p_afd, '.')
        pylab.subplot(313)
        pylab.plot(x_list, vx_list, '.')
        pylab.plot(x_list, v_exact, '.')
        pylab.plot(x_list, v_afd, '.')
        pylab.show()

    l1_density = l1_fit_factor(d_list, d_exact)
    l1_pressure = l1_fit_factor(p_list, p_exact)
    l1_velocity = l1_fit_factor(vx_list, v_exact)

    f = open('gradesheet.txt','w')
    f.write(str(l1_density)+'\n')
    f.write(str(l1_pressure)+'\n')
    f.write(str(l1_velocity)+'\n')
    f.close()
    
    return l1_density < 1.27e-5 and \
        l1_pressure < 2.2e-5 and \
        l1_velocity < 3.4e-5

import sys
if __name__=='__main__':
    print main()



# Generated with cmake-format 0.5.1
# How wide to allow formatted cmake files
line_width = 80

# How many spaces to tab for indent
tab_size = 2

# If arglists are longer than this, break them always
max_subargs_per_line = 10

# If true, separate flow control names from their parentheses with a space
separate_ctrl_name_with_space = False

# If true, separate function names from parentheses with a space
separate_fn_name_with_space = False

# If a statement is wrapped to more than one line, than dangle the closing
# parenthesis on its own line
dangle_parens = False

# What character to use for bulleted lists
bullet_char = '*'

# What character to use as punctuation after numerals in an enumerated list
enum_char = '.'

# What style line endings to use in the output.
line_ending = u'unix'

# Format command names consistently as 'lower' or 'upper' case
command_case = u'lower'

# Format keywords consistently as 'lower' or 'upper' case
keyword_case = u'unchanged'

# Specify structure for custom cmake functions
additional_commands = {
  "foo": {
    "flags": [
      "BAR",
      "BAZ"
    ],
    "kwargs": {
      "HEADERS": "*",
      "DEPENDS": "*",
      "SOURCES": "*"
    }
  }
}

# A list of command names which should always be wrapped
always_wrap = []

# Specify the order of wrapping algorithms during successive reflow attempts
algorithm_order = [0, 1, 2, 3, 4]

# If true, the argument lists which are known to be sortable will be sorted
# lexicographicall
autosort = False

# enable comment markup parsing and reflow
enable_markup = True

# If comment markup is enabled, don't reflow the first comment block in
# eachlistfile. Use this to preserve formatting of your
# copyright/licensestatements.
first_comment_is_literal = False

# If comment markup is enabled, don't reflow any comment block which matchesthis
# (regex) pattern. Default is `None` (disabled).
literal_comment_pattern = None

# Regular expression to match preformat fences in comments
# default=r'^\s*([`~]{3}[`~]*)(.*)$'
fence_pattern = u'^\\s*([`~]{3}[`~]*)(.*)$'

# Regular expression to match rulers in comments
# default=r'^\s*[^\w\s]{3}.*[^\w\s]{3}$'
ruler_pattern = u'^\\s*[^\\w\\s]{3}.*[^\\w\\s]{3}$'

# If true, emit the unicode byte-order mark (BOM) at the start of the file
emit_byteorder_mark = False

# If a comment line starts with at least this many consecutive hash characters,
# then don't lstrip() them off. This allows for lazy hash rulers where the first
# hash char is not separated by space
hashruler_min_length = 10

# If true, then insert a space between the first hash char and remaining hash
# chars in a hash ruler, and normalize its length to fill the column
canonicalize_hashrulers = True

# Specify the encoding of the input file. Defaults to utf-8.
input_encoding = u'utf-8'

# Specify the encoding of the output file. Defaults to utf-8. Note that cmake
# only claims to support utf-8 so be careful when using anything else
output_encoding = u'utf-8'

# A dictionary containing any per-command configuration overrides. Currently
# only `command_case` is supported.
per_command = {}



"""
Exact newtonian Riemann solver for an ideal gas
"""

def phugo(v2,d1,p1,g):
	"""
	Calculates the pressure along the hugoniot
	Input:
	v2 - Velocity jump
	d1 - Old density
	p1 - Old pressure
	g - Adiabatic index
	"""

	import math

	return (4*p1 + d1*(1 + g)*v2**2 + \
			math.sqrt(d1)*v2*\
			math.sqrt(16*g*p1 + \
					  d1*(1 + g)**2*v2**2))/4.

def vhugo(p2, d1, p1, g):

	"""
	Calculates the velocity along the hugoniot
	Input:
	p2 - New pressure
	d1 - Old density
	p1 - Old pressure
	g - Adiabatic index
	"""
	
	import math
	
	return (math.sqrt(2)*(-p1 + p2))/math.sqrt(d1*((-1 + g)*p1 + (1 + g)*p2))
	
def visen(p2, d1, p1, g):

	"""
	Calculates the velocity along the isentrope
	Input:
	p2 - New pressure
	d1 - Old density
	p1 - Old pressure
	g - Adiabatic index
	"""
	
	import math
	
	return (2*math.sqrt(g)*p1**(1/(2.*g))*(-p1**((-1 + g)/(2.*g)) + \
	p2**((-1 + g)/(2.*g))))/(math.sqrt(d1)*(-1 + g))
	
def vhydro(p2, d1, p1, g):

	"""
	Calculatest the velocity along the hydrodynamic trajectory
	Input:
	p2 - New pressure
	d1 - Old density
	p1 - Old pressure
	g - Adiabatic index
	"""
	
	if p2>p1:
		return vhugo(p2,d1,p1,g)
	else:
		return visen(p2,d1,p1,g)

class Primitive:
	"""
	Primitive variable (density, pressure and velocity)
	"""
	
	def __init__(self, d, p, v):
	
		"""
		Class constructor
		Input:
		d - Density
		p - Pressure
		v - Velocity
		"""
		
		self.Density = d
		self.Pressure = p
		self.Velocity = v
		
		return

	def clone(self):

		"""
		Creates a clone of the struct
		"""

		res = Primitive(self.Density,
				self.Pressure,
				self.Velocity)
		return res

def left_velocity(pnew, prim, g):
	
	"""
	Calculates the velocity on the left side of the interface
	Input:
	pnew - New pressure
	prim - Primitive variables
	g - Adiabatic index
	"""
	
	d1 = prim.Density;
	p1 = prim.Pressure;
	v1 = prim.Velocity;
	return v1 - vhydro(pnew, d1, p1, g)
		
def right_velocity(pnew, prim, g):
	
	"""
	Calculates the velocity on the right side of the interface
	Input:
	p2 - New pressure
	prim - Primitive variables
	g - Adiabatic index
	"""
	
	d1 = prim.Density
	p1 = prim.Pressure
	v1 = prim.Velocity
	return v1 + vhydro(pnew,d1,p1,g)
	
def eval_trans_eqn(p, left, right, g):

	"""
	Evaluates the transcendental equation
	Input:
	p - Pressure at the interface
	left - Primitive variables on the left side
	right - Primitive variables on the right side
	g - Adiabatic index
	"""
	
	return right_velocity(p,right,g) - left_velocity(p,left,g)
	
def find_upper_bracket(f, xl):

	"""
	Increases the range until the root is bracketed
	Input:
	f - Function
	xl - Lower boundary
	"""
	
	max_iter = 10
	iter = 0
	
	fl = f.Eval(xl)
	xr = xl
	fr = f.Eval(xr)
	while fl*fr>=0:
		xr = 2*xr
		fr = f.Eval(xr)
		if iter>max_iter:
			raise NameError('Maximum number of iteration exceeded in find_upper_bracket')
		else:
			iter = iter + 1
	return 2*xr
	
def bisection(f, xl, xr, tol):

	"""
	Solves an equation using the bisection method
	Input:
	f - Function
	lb - Left bound
	rb - Right bound
	tol - Tolerance
	"""
	
	max_iter = 100
	iter = 0
	
	fl = f.Eval(xl)
	if fl==0:
		return xl
	fr = f.Eval(xr)
	if fr==0:
		return xr
	
	if fl*fr>0:
		print('xl = '+str(xl))
		print('xr = '+str(xr))
		print('fl = '+str(fl))
		print('fr = '+str(fr))
		raise NameError("Root not bracketed")
	
	xm = 0.5*(xl+xr)
	while(abs(xl-xr)>tol):
		xm = 0.5*(xl+xr)
		fm = f.Eval(xm)
		if fm==0:
			return xm
		if fm*fl>0:
			xl = xm
		elif fm*fr>0:
			xr = xm
		else:
			raise NameError('Something bad happened. Probably a NaN')
			
		if iter>max_iter:
			raise NameError('Maximum number of iterations exceeded in bisection')
		else:
			iter = iter + 1
	
	return xm
	
class TransEqn:

	"""
	Transcendental equation
	"""
	
	def __init__(self, left, right, g):
	
		"""
		Class constructor
		Input:
		left - Primitive variables on the left side
		right - Primitive variables on the right side
		g - Adiabatic index
		"""
		
		self._left = left
		self._right = right
		self._g = g
		
		return
	
	def Eval(self, p):
	
		"""
		Evaluates the transcendental equation
		Input:
		p - Pressure at the interface
		"""
		
		return eval_trans_eqn(p,self._left, self._right, self._g)
		
def two_rarefaction(left, right, g):

	"""
	Pressure at the interface of a riemann problem in the case of two rarefactions
	Input:
	left - Primitive variables on the left side
	right - Primitive variables on the right side
	g - Adiabatic index
	"""
	
	import math
	
	dl = left.Density
	pl = left.Pressure
	vl = left.Velocity
	dr = right.Density
	pr = right.Pressure
	vr = right.Velocity
	ps = (4**(g/(1 - g))*(2*(math.sqrt(dr*g*pl) + math.sqrt(dl*g*pr)) + \
		(-1 + g)*math.sqrt(pl)*math.sqrt(pr)*(vl - vr))**((2*g)/(-1 + g)))/\
       (g**(g/(-1 + g))*(dr**(1/(2.*g))*math.sqrt(pl) + dl**(1/(2.*g))*math.sqrt(pr))**((2*g)/(-1 + g)))
	
	return ps

def two_strong_shocks(left, right, g):

	"""
	Pressure at the interface of the Riemann problem in the case of two strong shocks
	Input:
	left - Primitive variables on the left side
	right - Primitive variables on the right side
	g - Adiabatic index
	"""
	
	import math
	
	dl = left.Density
	vl = left.Velocity
	dr = right.Density
	vr = right.Velocity
	return (dl*dr*(1 + g)*(vl - vr)**2)/(2.*(math.sqrt(dl) + math.sqrt(dr))**2)
	
def acoustic_rp(left, right, g):

	"""
	Pressure at the interface of a riemann problem in the case of two acoustic waves
	Input:
	left - Primitive variable on the left side
	right - Primitive variable on the right side
	g - Adiabatic index
	"""
	
	import math
	
	dl = left.Density
	pl = left.Pressure
	vl = left.Velocity
	dr = right.Density
	pr = right.Pressure
	vr = right.Velocity
	return (math.sqrt(dl*g*pl)*pr + math.sqrt(dr*g*pr)*(pl + math.sqrt(dl*g*pl)*(vl - vr)))\
			/(math.sqrt(dl*g*pl) + math.sqrt(dr*g*pr))
	
def riemann_solve(left, right, g):

	"""
	Calculates the pressure and velocity at the interface
	Input:
	left - Primitive variables on the left side
	right - Primitive variable on the right side
	g - Adiabatic index
	"""
	
	tol = 1e-6
	
	vlpr = left_velocity(right.Pressure, left, g)
	vrpl = right_velocity(left.Pressure, right, g)
	if min(left.Velocity,vlpr)>max(right.Velocity,vrpl):
		# Two shocks
		eqn = TransEqn(left, right, g)
		p1 = max(left.Pressure, right.Pressure)
		plvr = phugo(left.Velocity-right.Velocity,
			     left.Density,left.Pressure,g)
		prvl = phugo(left.Velocity-right.Velocity,
			     right.Density,right.Pressure,g)
		p2 = max(plvr,prvl)
		ps = bisection(eqn,p1,p2,tol)
	elif max(left.Velocity,vlpr)<min(right.Velocity,vrpl):
		# Two rarefactions
		ps = two_rarefaction(left, right, g)
	else:
		# Shock rarefaction
		eqn = TransEqn(left, right, g)
		ps = bisection(eqn, min(left.Pressure, right.Pressure), max(left.Pressure, right.Pressure), tol)
	vs = 0.5*(left_velocity(ps,left,g)+right_velocity(ps,right,g))
	return ps, vs
	
# Self similar spatial profiles

# shocks

def shock_speed(p2, d1, p1, g):

	"""
	Calculates the shock speed in the upstream reference frame
	Input:
	p2 - Downstream pressure
	d1 - Upstream density
	p1 - Upstream pressure
	g - Adiabatic index
	"""
	
	import math
	
	return math.sqrt(-p1 + g*p1 + p2 + g*p2)/(math.sqrt(2)*math.sqrt(d1))
	
def dhugo(p2,d1,p1,g):

	"""
	Calculates the density on the hugoniot curve
	Input:
	p2 - Downstream pressure
	d1 - Upstream density
	p1 - Upstream pressure
	g - Adiabatic index
	"""
	
	return (d1*((-1 + g)*p1 + (1 + g)*p2))/((1 + g)*p1 + (-1 + g)*p2)

class ShockProf:
	"""
	Spatial profile of a shock wave
	"""

	def __init__(self, p2, d1, p1, g):
		"""
		Class constructor
		Input:
		p2 - Downstream pressure
		d1 - Upstream density
		p1 - Upstream pressure
		g - Adiabatic index
		"""
		
		self._vs = shock_speed(p2,d1,p1,g)
		self._upstream = Primitive(d1, p1, 0)
		self._downstream = Primitive(\
			dhugo(p2,d1,p1,g),\
				p2, \
				vhugo(p2,d1,p1,g))

		return
		
	def CalcPrim(self, v):
	
		"""
		Retrives the primitive variables
		Input:
		v - Dimensionless coordinate (v=x/t)
		"""
		
		if(v>self._vs):
			return Primitive(self._upstream.Density,\
							self._upstream.Pressure,\
							self._upstream.Velocity)
		else:
			return Primitive(self._downstream.Density,\
							self._downstream.Pressure,\
							self._downstream.Velocity)
			
# Isentropes

def disen(p2,d1,p1,g):

	"""
	Density on the isentrope
	Input:
	p2 - Downstream pressure
	d1 - Upstream density
	p1 - Upstream pressure
	g - Adiabatic index
	"""
	
	return d1*(p2/p1)**(1./g)
	
def visen(p2,d1,p1,g):

	"""
	Calculates the velocity on the hugoniot
	Input:
	p2 - Downstream pressure
	d1 - Upstream density
	p1 - Upstream pressure
	g - Adiabatic index
	"""
	
	import math
	
	return (2*math.sqrt(g)*p1**(1/(2.*g))*(-p1**((-1 + g)/(2.*g)) +\
			p2**((-1 + g)/(2.*g))))/(math.sqrt(d1)*(-1 + g))
	
def sound_speed(p, d, g):

	"""
	Calculates the speed of sound
	Input:
	p - Pressure
	d - Density
	g - Adiabatic index
	"""
	
	import math
	
	return math.sqrt(g*p/d)
	
class IsenProf:
	"""
	Spatial profile of a rarefaction wave
	"""
	
	def __init__(self,p2,d1,p1,g):
		"""
		Class constructor
	
		"""
		
		self._p1 = p1
		self._d1 = d1
		self._g = g
		self._upstream = Primitive(d1,p1,0)
		self._csmax = sound_speed(p1,d1,g)
		d2 = disen(p2,d1,p1,g)
		v2 = visen(p2,d1,p1,g)
		self._csmin = sound_speed(p2,d2,g)+v2
		self._downstream = Primitive(d2,p2,v2)
	
		return
	
	def CalcPrim(self, v):
		
		"""
		Calculates the primitive variables
		Input:
		v - Dimensionless coordinate (x/t)
		"""
		
		if v > self._csmax:
			return Primitive(self._upstream.Density,\
							self._upstream.Pressure,\
							self._upstream.Velocity)
		elif v < self._csmin:
			return Primitive(self._downstream.Density,\
							self._downstream.Pressure,\
							self._downstream.Velocity)
		else:

			u = (v-self._csmax)*2/(1+self._g)
			c = self._csmax+(self._g-1)*u/2;
			p = self._p1*(c/self._csmax)**(2*self._g/(self._g-1))
			d = disen(p,self._d1,self._p1,self._g)
			return Primitive(d,p,u)
			
class HydroProf:
	"""
	Spatial profile for both shock and rarefaction
	"""
	
	def __init__(self, p2, d1, p1, g):
		
		"""
		Class constructor
		Input:
		p2 - Downstream pressure
		d1 - Upstream density
		p1 - Upstream pressure
		g - Adiabatic index
		"""
		
		if p2>p1:
			self._prof = ShockProf(p2,d1,p1,g)
		else:
			self._prof = IsenProf(p2,d1,p1,g)
		
	def CalcPrim(self, v):
		
		"""
		Calculates the primitive variables
		"""
		
		return self._prof.CalcPrim(v)
		
class RiemannProfile:
	
	"""
	Complete Riemann problem profile
	"""
	
	def __init__(self, left, right, g):
		
		"""
		Class constructor
		Input:
		left - Primitive variables on the left side
		right - Primitive variables on the right side
		g - Adiabatic index
		"""
		
		self._ps, self._vs = riemann_solve(left, right, g)
		self._left_prof = HydroProf(self._ps,left.Density,\
						    left.Pressure, g)
		self._right_prof = HydroProf(self._ps,right.Density,\
						     right.Pressure,g)
		self._left = left
		self._right = right
		
		return
	
	def CalcPrim(self, v):
		
		"""
		Calculates the primitive variables
		Input:
		v - Self similar coordinate (x/t)
		"""
		
		if v>self._vs:
			res = self._right_prof.CalcPrim(v - self._right.Velocity)
			res.Velocity = res.Velocity + self._right.Velocity
		else:
			res = self._left_prof.CalcPrim(self._left.Velocity - v)			
			res.Velocity = self._left.Velocity-res.Velocity
		return res



#! /usr/bin/python

def chi_squared(a1, a2):

    import math

    diff2 = [(x-y)**2 for x,y in zip(a1,a2)]
    return math.sqrt(sum(diff2)/len(a1))

def L1_error_norm(a1,a2):

    import math
    dfrnc = [abs(x-y) for x,y in zip(a1,a2)]
    return sum(dfrnc)/len(a1)

def error_norm(a1,a2,test_name='L1'):

    if('L1'==test_name):
        return L1_error_norm(a1,a2)
    elif('chi2'==test_name):
        return chi_squared(a1,a2)
    else:
        raise NameError('unknown test type '+test_name)

def main():

    import os
    import imp
    enrs = imp.load_source('enrs','../../analytic/enrs.py')

    import numpy

    left = enrs.Primitive(1,1,-1);
    right = enrs.Primitive(1,1,1);
    prof = enrs.RiemannProfile(left,right,5./3.)
    t = numpy.loadtxt('time.txt')
    offset = 0.5

    x = numpy.loadtxt('cell_centres.txt')
    d = numpy.loadtxt('densities.txt')
    p = numpy.loadtxt('pressures.txt')
    v = numpy.loadtxt('velocities.txt')
    da = [prof.CalcPrim((i-offset)/t).Density for i in x]
    pa = [prof.CalcPrim((i-offset)/t).Pressure for i in x]
    va = [prof.CalcPrim((i-offset)/t).Velocity for i in x]

    test_name = 'L1'
    gof1 = error_norm(d,da,test_name)
    gof2 = error_norm(p,pa,test_name)
    gof3 = error_norm(v,va,test_name)

    f = open('gradesheet.txt','w')
    f.write(str(gof1)+'\n')
    f.write(str(gof2)+'\n')
    f.write(str(gof3)+'\n')
    f.close()

    return gof1<0.06 and gof2<0.02 and gof3 < 0.04

import sys
if __name__=='__main__':
    print main()



def run_single_test(fpath):

    import os
    import run_tests as nrt
    import imp
    homebrew_makefile = imp.load_source(\
        'homebrew_makefile',
        '../homebrew_makefile.py')

    tfname = nrt.temp_folder_name(fpath)
    nrt.new_empty_directory(tfname)
    os.system('cp '+
              os.path.join(fpath+'/*').replace('//','/')+' '+
              tfname)

    # Compile
    os.system('g++ -o '+tfname+'/test.o '+
              '-c -O3 '+
              homebrew_makefile.gcc_strict_warning_flags()+' '+
              (fpath+'/test.cpp').replace('//','/')+' '+
              '-I ..')

    # Link
    hdf5_lib_path = os.environ['HDF5_LIB_PATH']
    os.system('g++ -o '+tfname+'/test.exe '+
              tfname+'/test.o '+
              '../library/librich.a '+
              (hdf5_lib_path+'/libhdf5_cpp.a').replace('//','/')+' '+
              (hdf5_lib_path+'/libhdf5_hl.a').replace('//','/')+' '+
              (hdf5_lib_path+'/libhdf5.a').replace('//','/')+' '+
              '-lpthread -lz -ldl ')

    # Run
    os.chdir(tfname)
    os.system('./test.exe')
    import imp
    mod = imp.load_source('mod',os.getcwd()+'/test.py')
    res = mod.main()
    os.chdir('..')
    return res

def run_single_test_weh(fpath, home_dir):

    import os
    
    try:
        tf = run_single_test(fpath)
        if tf:
            return fpath+' ... passed'
        else:
            return fpath+' ... ran and failed'
    except Exception, err:
        print err
        os.chdir(home_dir)
        return fpath+' ... failed due to error'

def get_all_file_paths():

    import os

    res = []
    for root, dirs, files in os.walk(''):
        for name in files:
            res.append(os.path.join(root, name))
    return res

def get_source_paths():

    return [fname for fname in get_all_file_paths()
            if 'test.cpp' in fname and not '.svn' in fname]

def get_test_paths():

    return [fpath.replace('/test.cpp','') for fpath in get_source_paths() if not '~' in fpath]

def main():

    import multiprocessing
    import os
    import run_tests as nrt

    nrt.build_library()

    msgs = []

    pool = multiprocessing.Pool()
    [pool.apply_async(run_single_test_weh,
                      (l,os.getcwd()),
                      callback=msgs.append)
     for l in get_test_paths()]
    pool.close()
    pool.join()

    f = open('test_report.txt','w')
    for i in msgs:
        f.write(i+'\n')
    f.close()

if __name__ == '__main__':
    
    main()



#! /usr/bin/python

def goodness_of_fit(a1, a2):

    import math

    diff2 = [(x-y)**2 for x,y in zip(a1,a2)]
    return math.sqrt(sum(diff2)/(max(a1)-min(a1))**2/len(a1))

def main():

    import os
    import imp
    enrs = imp.load_source('enrs',os.environ['RICH_ROOT']+'/analytic/enrs.py')

    import numpy
    import h5py

    left = enrs.Primitive(1,10,0);
    right = enrs.Primitive(1,1,0);
    prof = enrs.RiemannProfile(left,right,5./3.)
    h5f = h5py.File('final.h5')
    t = h5f['time']
    offset = 0.5

    x_list = h5f['grid']
    numeric = {}
    numeric['density'] = h5f['density']
    numeric['pressure'] = h5f['pressure']
    numeric['velocity'] = h5f['x_velocity']
    analytic = {}
    analytic['density'] = \
        [prof.CalcPrim((i-offset)/t).Density for i in x_list]
    analytic['pressure'] = \
        [prof.CalcPrim((i-offset)/t).Pressure for i in x_list]
    analytic['velocity'] = \
        [prof.CalcPrim((i-offset)/t).Velocity for i in x_list]

    gof = {}
    for varname in analytic:
        gof[varname] = \
            goodness_of_fit(numeric[varname],
                            analytic[varname])

    f = open('gradesheet.txt','w')
    for varname in ['density', 'pressure', 'velocity']:
        f.write(str(gof[varname])+'\n')
    f.close()

    return gof['density'] < 0.11 and \
        gof['pressure'] < 0.05 and \
        gof['velocity'] < 0.11

if __name__=='__main__':

    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')




def prep_dif_res_runs(path_to_exec,res_list,res_file_name='resolution.txt'):
    """
    Prepares a series of the same simulation run with different resolutions
    It assumes the resolution is read off from a text file "resolutions.txt"
    Input:
    path_to_exec - Path to folder with executable
    res_list - List of resolutions to be scanned
    res_file_name - Name of the file were the resolutions is to be written
    """

    if '/' in path_to_exec:
        raise NameError('No slashes expected in path_to_exec')
    
    import os
    for res in res_list:
        folder_name = path_to_exec+'_res_'+str(res)
        os.system('mkdir '+folder_name)
        os.system('cp '+path_to_exec+'/test.exe '+folder_name)
        os.system('cp '+path_to_exec+'/test.py '+folder_name)
        os.system('cp '+path_to_exec+'/*.txt '+folder_name)
        
        f = open(folder_name+'/'+res_file_name,'w')
        f.write(str(res)+'\n')
        f.close()

def check_valid_input(smallest_resolusion, largest_resolution):

    if largest_resolution<=smallest_resolusion:
        raise NameError('smallest resolution is larger than largest resolusion')
    if largest_resolution >= 1e5:
        raise NameError('Resolution is too high')

def main():

    import argparse
    import numpy
    import re

    parser = argparse.ArgumentParser(description='Creates simulation folders of the same calculation with different resolutions')
    parser.add_argument('test_folder',
                        help='path to test folder')
    parser.add_argument('lowest_res',
                        type=int,
                        help='lowest res (power of two)')
    parser.add_argument('highest_res',
                        type=int,
                        help='highest res (power of two)')
    args = parser.parse_args()

    check_valid_input(args.lowest_res, args.highest_res)

    resolution_list = [2**x for x in range(args.lowest_res,
                                           args.highest_res+1)]

    folder_name = re.sub(r'/$','',args.test_folder)
    if '/' in folder_name:
        raise NameError('Expected folder name to be in the same level as the script')

    prep_dif_res_runs(folder_name, resolution_list)

if __name__=='__main__':

    main()



#! /usr/bin/python

import math

def goodness_of_fit(a1, a2):

    import math

    diff2 = [(x-y)**2 for x,y in zip(a1,a2)]
    return math.sqrt(sum(diff2)/(max(a1)-min(a1))**2/len(a1))

def density_prof(x):

    return 1+1e-6*math.sin(2*math.pi*x)

def pressure_prof(x):

    return 3./5.+1e-6*math.sin(2*math.pi*x)

def velocity_prof(x):

    return 1e-6*math.sin(2*math.pi*x)

def main():

    import os
    import numpy
    import h5py

    h5f = h5py.File('final.h5')
    x_list = h5f['grid']
    d_list = h5f['density']
    p_list = h5f['pressure']
    v_list = h5f['x_velocity']
    da = [density_prof(x) for x in x_list]
    pa = [pressure_prof(x) for x in x_list]
    va = [velocity_prof(x) for x in x_list]

    if False:
        import pylab

        pylab.subplot(311)
        pylab.plot(x_list,d_list,x_list,da)

        pylab.subplot(312)
        pylab.plot(x_list,p_list,x_list,pa)

        pylab.subplot(313)
        pylab.plot(x_list,v_list,x_list,va)

        pylab.show()

    gof1 = goodness_of_fit(d_list,da)
    gof2 = goodness_of_fit(p_list,pa)
    gof3 = goodness_of_fit(v_list,va)

    f = open('gradesheet.txt','w')
    f.write(str(gof1)+'\n')
    f.write(str(gof2)+'\n')
    f.write(str(gof3)+'\n')
    f.close()

    return gof1<0.06 and gof2<0.02 and gof3 < 0.04

if __name__=='__main__':

    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')




def build_library():

    import os

    os.chdir('..')
    os.system('python ./homebrew_makefile.py')
    os.chdir('convergence')

def temp_folder_name(test_path):

    import re
    
    res = 'temp_'+test_path
    res = res.replace('.','')
    res = res.replace('/','_')
    res = res.replace('__','_')
    res = re.sub(r'_$','',res)
    return res

def new_empty_directory(dirname):

    import os

    os.system('touch '+dirname)
    os.system('rm -rf '+dirname)
    os.system('mkdir '+dirname)

def main():

    import argparse
    import os
    import imp
    homebrew_makefile = imp.load_source(\
        'homebrew_makefile',
        '../homebrew_makefile.py')

    build_library()

    parser = argparse.ArgumentParser()
    parser.add_argument('test_folder',
                        help='path to test folder')
    parser.add_argument('-jb','--just_build',
                        action='store_true',
                        help='Only builds the test, but does not run it')
    args = parser.parse_args()
    
    tfname = temp_folder_name(args.test_folder)
    new_empty_directory(tfname)
    os.system('cp '+
              os.path.join((args.test_folder+'/*').replace('//','/'))+' '+
              tfname)

    # Compile
    os.system('g++ -o '+tfname+'/test.o '+
              '-c -O3 '+
              homebrew_makefile.gcc_strict_warning_flags()+' '+
              (args.test_folder+'/test.cpp').replace('//','/')+' '+
              '-I ..')
    
    # Link
    hdf5_lib_path = os.environ['HDF5_LIB_PATH']
    os.system('g++ -o '+tfname+'/test.exe '+
              tfname+'/test.o '+
              '../library/librich.a '+
              (hdf5_lib_path+'/libhdf5_cpp.a').replace('//','/')+' '+
              (hdf5_lib_path+'/libhdf5_hl.a').replace('//','/')+' '+
              (hdf5_lib_path+'/libhdf5.a').replace('//','/')+' '+
              '-lpthread -lz -ldl ')

    if not args.just_build:
        os.chdir(tfname)
        os.system('./test.exe')
        import imp
        mod = imp.load_source('mod',os.getcwd()+'/test.py')
        try:
            if mod.main():
                print args.test_folder+' ... passed'
            else:
                print args.test_folder+' ... ran and failed'
        except:
            print args.test_folder+' ... failed to run'
        os.chdir('..')        

if __name__ == '__main__':

    main()



#!/usr/bin/env python3
# -*- Coding: UTF-8 -*-

# ---------------------------------------------------------------------------
# Open Asset Import Library (ASSIMP)
# ---------------------------------------------------------------------------
#
# Copyright (c) 2006-2020, ASSIMP Development Team
#
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, 
# with or without modification, are permitted provided that the following 
# conditions are met:
# 
# * Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
# 
# * Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
# 
# * Neither the name of the ASSIMP team, nor the names of its
#   contributors may be used to endorse or promote products
#   derived from this software without specific prior
#   written permission of the ASSIMP Development Team.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# ---------------------------------------------------------------------------

import sys

Entity_token = "ENTITY"
Type_token   = "TYPE"
token = []
file = open(sys.argv[1])
output = open("step_entitylist.txt", "a")
lines = file.readlines()
for line in lines:
    pos = line.find(Entity_token)
    if pos != -1:
        token = line.split(" ")
        if len(token) > 1:
            name = token[1]
            print( "Writing entity " + name)
            output.write(name)

output.close()
file.close()


    



def calc_error_norm(a1,a2):

    abs_dif = [abs(x-y) for x,y in zip(a1,a2)]
    return sum(abs_dif)/len(a1)

def read_x_prof(fname):

    import numpy
    rawd = numpy.loadtxt(fname)
    return {'coordinate': rawd[:,0],
            'density': rawd[:,1],
            'pressure': rawd[:,2],
            'velocity': rawd[:,3],
            'sound speed': rawd[:,4]}

def calc_sound_speed(d,p,g):

    import math

    return math.sqrt(g*p/d)

class SineWave:
    """
    Sine wave spatial profile
    """

    def __init__(self,
                 amplitude,
                 wavelength,
                 phase,
                 offset):

        import math

        self.amp = amplitude
        self.k = 2*math.pi/float(wavelength)
        self.ph = phase
        self.offset = offset

    def eval(self, x):

        import math

        amp = self.amp
        k = self.k
        ph = self.ph
        offset = self.offset
        return amp*math.sin(k*x+ph)+offset

class SpatProf:

    """
    Acoustic spatial profiles for this test
    """

    def __init__(self, d0, p0, g,
                 dd, l):

        c0 = calc_sound_speed(d0,p0,g)
        self.c0 = c0
        self.density = SineWave(dd,l,0,d0)
        self.pressure = SineWave(dd*c0**2,
                                 l,0,p0)
        self.velocity = SineWave(dd*c0/d0,
                                 l,0,0)

def main():

    graphic_flag = False
    if graphic_flag:
        import pylab

    import numpy
    import h5py

    initial = h5py.File('initial.h5')
    final_data = h5py.File('final.h5')

    time = numpy.array(final_data['time'])[0]
    spat_prof = SpatProf(numpy.loadtxt('ambient_density.txt'),
                         numpy.loadtxt('ambient_pressure.txt'),
                         numpy.loadtxt('adiabatic_index.txt'),
                         numpy.loadtxt('amplitude.txt'),
                         numpy.loadtxt('width.txt'))
    analytic = dict()
    analytic['density'] = [spat_prof.density.eval(x-spat_prof.c0*time) \
                            for x in final_data['geometry']['x_coordinate']]
    analytic['pressure'] = [spat_prof.pressure.eval(x-spat_prof.c0*time) \
                            for x in final_data['geometry']['x_coordinate']]
    analytic['x_velocity'] = [spat_prof.velocity.eval(x-spat_prof.c0*time) \
                            for x in final_data['geometry']['x_coordinate']]

    if graphic_flag:
        for n, field in enumerate(['density','pressure','x_velocity']):
            pylab.subplot(3,1,n+1)
            pylab.plot(final_data['x_coordinate'],
                       final_data[field],'.')
            pylab.plot(final_data['x_coordinate'],
                       analytic[field],'.')
            pylab.plot(initial['x_coordinate'],
                       initial[field], '.')
        pylab.show()

    error_norm = dict()
    for i in analytic.keys():
        error_norm[i] = calc_error_norm(
            final_data['hydrodynamic'][i],analytic[i])

    f = open('gradesheet.txt','w')
    for i in analytic.keys():
        f.write(str(error_norm[i])+'\n')
    f.close()

    return error_norm['density']<5e-8 and \
        error_norm['pressure']<5e-8 and \
        error_norm['x_velocity']<5e-8

if __name__=='__main__':
    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')



#! /usr/bin/python

import matplotlib
matplotlib.use('Qt4Agg')
import pylab
import numpy
import sys
import imp
enrs = imp.load_source('enrs','../analytic/enrs.py')

pref = sys.argv[0].replace('draw_profiles.py','')

left = enrs.Primitive(1,1,-1);
right = enrs.Primitive(1,1,1);
prof = enrs.RiemannProfile(left,right,5./3.)
t = numpy.loadtxt(pref+'time.txt')
offset = 0.5

x = numpy.loadtxt(pref+'cell_centres.txt')
d = numpy.loadtxt(pref+'densities.txt')
p = numpy.loadtxt(pref+'pressures.txt')
v = numpy.loadtxt(pref+'velocities.txt')
da = [prof.CalcPrim((i-offset)/t).Density for i in x]
pa = [prof.CalcPrim((i-offset)/t).Pressure for i in x]
va = [prof.CalcPrim((i-offset)/t).Velocity for i in x]

pylab.subplot(311)
pylab.plot(x,d,x,da)
pylab.xlabel('x')
pylab.ylabel('Density')

pylab.subplot(312)
pylab.plot(x,p,x,pa)
pylab.xlabel('x')
pylab.ylabel('Pressure')

pylab.subplot(313)
pylab.plot(x,v,x,va)
pylab.xlabel('x')
pylab.ylabel('Velocity')

pylab.show()



def unstructured_contour(x_list, y_list, z_list):

    from matplotlib.tri import Triangulation
    from scipy.spatial import Delaunay
    import matplotlib.pyplot as plt
    import numpy

    tri = Delaunay(numpy.array([[x,y] for x,y
                                in zip(x_list, y_list)]))
    mtri = Triangulation(x_list, y_list, tri.vertices.copy())
    res = plt.tricontourf(mtri, z_list)
    return res

def consolidate_data(fname, g = 5./3.):

    import h5py
    import numpy

    f = h5py.File(fname)
    res = {}
    for field in f:
        res[field] = numpy.array(f[field])
    res['log_density'] = numpy.log(res['density'])
    res['log_pressure'] = numpy.log(res['pressure'])
    res['sound_speed'] = numpy.sqrt(res['pressure']/res['density'])
    res['speed'] = numpy.sqrt(res['x_velocity']**2+res['y_velocity']**2)
    res['mach_number'] = res['speed']/res['sound_speed']
    return res

def collect_snapshot_data(file_list):

    import numpy

    part_list = [consolidate_data(fname) for fname
                 in file_list]
    res = {}
    for field in part_list[0]:
        res[field] = numpy.concatenate([itm[field] for itm in part_list])
    return res

def main():

    import argparse
    import glob
    import pylab

    parser = argparse.ArgumentParser(description='Plots RICH snapshots')
    parser.add_argument('pattern',help='Pattern to match all output files')
    parser.add_argument('varname',help='name of variable to be plotted (z axis)')
    args = parser.parse_args()

    data = collect_snapshot_data(glob.glob(args.pattern))
    unstructured_contour(data['x_coordinate'],
                         data['y_coordinate'],
                         data[args.varname])
    pylab.colorbar()
    pylab.show()

if __name__ == '__main__':

    main()

    
